/*
 * CrossSection.java
 *
 * Created on March 13, 2007, 9:25 AM
 *
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;

//import java.io.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;
import java.util.Enumeration;
import java.util.ListIterator;
import javax.swing.JPanel;


/**
 *
 * This class represents a cross-section of a rock. It contains base (i.e. non-
 * transformed) elipses and background image (if one exists). It also contains
 * the routines necessary for data persistence (via a file or files).
 * @author cwarren
 */
public class CrossSection
{
    // the config / source data for this cross section
    private Properties srcData;
    
    // the background image
    private BufferedImage origBgImage;
    private BufferedImage bgImage;
    
    private String bgImageFile;
    
    // the cross that's used when there's no BG image displyed
    private BufferedImage origBgCrossImage;
    private BufferedImage bgCrossImage;
    
    // the image on which all the pebbles and the BG are shown
    private BufferedImage crossSectionImage;
    
    private AffineTransform currentDeformation;
    
    // the pebbles in this cross section
    private ArrayList pebbleList;
    
    public static StrainPebble STRAIN_ELLIPSE_BASE  = new StrainPebble(350,350,300,300,0,Color.LIGHT_GRAY);
    private StrainPebble strainEllipse; // = CrossSection.STRAIN_ELLIPSE_BASE.clone ();
    
    // long and short axis ratios
    private double maxLongRatio;
    private double maxShortRatio;
    
    // transform state info
    private double shearX = 0;
    private double shearY = 0;
    
    private double shearAngle = 0;
    private double shearAmount = 0;
    
    private double offsetX = 0;
    private double offsetY = 0;
    
    private double scaleX = 1.0;
    private double scaleY = 1.0;
    
    private double magnifierScale = 1.0;
    
    private double displayAngle = 0;
    
    // limits to how much the cross section can be distorted
    public static double MAX_SHEAR = 5.0;
    
    
    public static double MAX_SHEAR_X = 5.0;
    public static double MIN_SHEAR_X = -5.0;
    public static double MAX_SHEAR_Y = 5.0;
    public static double MIN_SHEAR_Y = -5.0;
    
    public static double MAX_SCALE_X = 5.0;
    public static double MIN_SCALE_X = 0.2;
    public static double MAX_SCALE_Y = 5.0;
    public static double MIN_SCALE_Y = 0.2;
    
    public static double MAX_MAGNIFIER = 2.5;
    public static double MIN_MAGNIFIER = 0.1;
    
    public static int DIMX = 700;
    public static int DIMY = 700;
    
    private Color bgColor = Color.WHITE;
    
    // various display flags
    private boolean pebbleShowAxesState = false;
    private boolean pebbleFillState = false;
    private boolean showBackgroundImage = true;
    
    private boolean adminMode = false;
    private boolean oneOrMorePebblesSelected = false;
    
    private ArrayList watchingPlotters;
    
    //**************************************************************************
    // CONSTRUCTORS
    //**************************************************************************
    
    /** Creates a new instance of CrossSection */
    public CrossSection ()
    {
        this.init ();
    }
    
    private void init()
    {
        this.strainEllipse = CrossSection.STRAIN_ELLIPSE_BASE.clone ();
        
        this.setCrossSectionImage (new BufferedImage (this.DIMX, this.DIMY, BufferedImage.TYPE_INT_RGB));
        this.srcData = new Properties ();
        this.pebbleList = new ArrayList ();
        
        this.setShearX (0.0);
        this.setShearY (0.0);
        
        this.origBgCrossImage = new BufferedImage (this.DIMX, this.DIMY, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2D = this.origBgCrossImage.createGraphics ();
        g2D.setColor (Color.WHITE);
        g2D.fillRect (0,0,this.DIMX,this.DIMY);
        g2D.setColor (Color.LIGHT_GRAY);
        g2D.drawLine (this.DIMX/2,0,this.DIMX/2,this.DIMY);
        g2D.drawLine (0,this.DIMY/2,this.DIMX,this.DIMY/2);
        
        this.bgCrossImage = new BufferedImage (this.DIMX, this.DIMY, BufferedImage.TYPE_INT_RGB);
        
        // load the test file into the srcData properties
        //String filename = "C:\\geoshear_pebbles.txt";
        //this.loadFromFile (filename);
        
        this.watchingPlotters = new ArrayList (1);        
    }
    
    private void setInitialValues()
    {
        this.strainEllipse = CrossSection.STRAIN_ELLIPSE_BASE.clone ();        
        this.setCrossSectionImage (new BufferedImage (this.DIMX, this.DIMY, BufferedImage.TYPE_INT_RGB));        
        this.setShearX (0.0);
        this.setShearY (0.0);
        this.setScaleX (1.0);
        this.setScaleY (1.0);
        this.setMagnifierScale (1.0);
        this.setDisplayAngle (0.0);
        this.setAdminMode (false);
        this.setCurrentDeformation (null);
        this.setOffsetX (0.0);
        this.setOffsetY (0.0);
        this.setOneOrMorePebblesSelected (false);
        this.setPebbleAxesState (false);
        this.setPebbleFillState (false);
        this.setPebbleShowAxesState (false);
        this.setShearAmount (0.0);
        this.setShearAngle (0.0);
        this.setShowBackgroundImage (true);        
        this.origBgCrossImage = new BufferedImage (this.DIMX, this.DIMY, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2D = this.origBgCrossImage.createGraphics ();
        g2D.setColor (Color.WHITE);
        g2D.fillRect (0,0,this.DIMX,this.DIMY);
        g2D.setColor (Color.LIGHT_GRAY);
        g2D.drawLine (this.DIMX/2,0,this.DIMX/2,this.DIMY);
        g2D.drawLine (0,this.DIMY/2,this.DIMX,this.DIMY/2);        
        this.bgCrossImage = new BufferedImage (this.DIMX, this.DIMY, BufferedImage.TYPE_INT_RGB);
    }
    
    //**************************************************************************
    // OBJECT PERSISTENCE METHODS
    //**************************************************************************
    
    public void loadFromFile (String fileName)
    {
        try
        {
            // this is a test to make sure the load works - if it fails then the exception happens
            this.srcData.load (new FileInputStream (fileName));

            // now we actually clear and load the data
            this.setBgImage (null);
            this.srcData.clear ();            
            this.srcData.load (new FileInputStream (fileName));
            
            this.setInitialValues ();
            
        }
        catch (FileNotFoundException exc)
        {
            System.err.println (exc.toString ());
            System.exit (1);
        }
        catch (IOException exc)
        {
            System.err.println (exc.toString ());
            System.exit (1);
        }
        
        this.updateFromSrcData ();
        this.strainEllipse = (StrainPebble)(CrossSection.STRAIN_ELLIPSE_BASE.clone ());
        this.setupCrossSectionImage ();
    }
    
    public void loadFromFile (File f)
    {
        this.loadFromFile (f.getAbsolutePath ());
    }
    
    public void storeToFile (String fileName)
    {
        this.updateSrcData ();
        
        try
        {
            this.srcData.store (new FileOutputStream (fileName),"");
        }
        catch (FileNotFoundException ex)
        {
            ex.printStackTrace ();
        }
        catch (IOException ex)
        {
            ex.printStackTrace ();
        }
    }
    
    public void storeToFile (File f)
    {
        this.storeToFile (f.getAbsolutePath ());
    }
    
    public void storeDeformedToFile (String fileName)
    {
        this.updateDeformedSrcData ();
        
        try
        {
            this.srcData.store (new FileOutputStream (fileName),"");
        }
        catch (FileNotFoundException ex)
        {
            ex.printStackTrace ();
        }
        catch (IOException ex)
        {
            ex.printStackTrace ();
        }
    }

    public void storeDeformedToFile (File f)
    {
        this.storeDeformedToFile (f.getAbsolutePath ());
    }

    // save to a file as tab delimited text, so the data can easily be read
    // by other programs
    public void exportToFile (String fileName)
    {
        this.updateSrcData ();

        try
        {
            PrintStream pfout = new PrintStream(new FileOutputStream (fileName));

            pfout.println ("GeoShear data export");
            pfout.println ("");
            pfout.println ("ID\tCenterX\tCenterY\tShortAxis\tLongAxis\tAngle(deg)");
                        
            Object[] pebsArray = this.pebbleList.toArray ();
            Arrays.sort (pebsArray, new PebbleComparator());
            ListIterator pli = Arrays.asList (pebsArray).listIterator ();
            
            int pebNum = 0;
            while (pli.hasNext ())
            {
                pebNum++;
                Pebble tempPeb = (Pebble)(pli.next ());
                pfout.println ("pebble_"+Utils.rightmost ("000"+pebNum,3)
                        +"\t"+tempPeb.getOrigCenter ().x
                        +"\t"+tempPeb.getOrigCenter ().y
                        +"\t"+tempPeb.getOrigShortAxisLength ()
                        +"\t"+tempPeb.getOrigLongAxisLength ()
                        +"\t"+tempPeb.getOrigThetaDeg ());
            }
                        
            pfout.close ();
        }
        catch (FileNotFoundException ex)
        {
            ex.printStackTrace ();
        }
        catch (IOException ex)
        {
            ex.printStackTrace ();
        }
    }
    
    public void exportToFile (File f)
    {
        this.exportToFile (f.getAbsolutePath ());
    }
    
    public void exportDeformedToFile (String fileName)
    {
        this.updateDeformedSrcData ();
        
        try
        {
            PrintStream pfout = new PrintStream(new FileOutputStream (fileName));

            pfout.println ("GeoShear data export");
            pfout.println ("");
            pfout.println ("ID\tCenterX\tCenterY\tShortAxis\tLongAxis\tAngle(deg)");
                        
            Object[] pebsArray = this.pebbleList.toArray ();
            Arrays.sort (pebsArray, new PebbleComparator());
            ListIterator pli = Arrays.asList (pebsArray).listIterator ();
            
            int pebNum = 0;
            while (pli.hasNext ())
            {
                pebNum++;
                Pebble tempPeb = (Pebble)(pli.next ());
                pfout.println ("pebble_"+Utils.rightmost ("000"+pebNum,3)
                        +"\t"+tempPeb.getCenter ().x
                        +"\t"+tempPeb.getCenter ().y
                        +"\t"+tempPeb.getShortAxisLength ()
                        +"\t"+tempPeb.getLongAxisLength ()
                        +"\t"+tempPeb.getThetaDeg ());
            }
                        
            pfout.close ();
        }
        catch (FileNotFoundException ex)
        {
            ex.printStackTrace ();
        }
        catch (IOException ex)
        {
            ex.printStackTrace ();
        }
    }

    public void exportDeformedToFile (File f)
    {
        this.exportDeformedToFile (f.getAbsolutePath ());
    }
    
    //**************************************************************************
    // OTHER 'REAL' METHODS
    //**************************************************************************
    
    // updates the src data properties list from the current pebble list and bg image
    // used before storing to make sure everything is saved.
    // NOTE: current deformations are NOT saved!
    private void updateSrcData ()
    {
        this.srcData.clear ();
        if ((this.getBgImageFile () != null) && (this.getBgImageFile ().length () > 0))
        {
            this.srcData.put ("bgimage",this.getBgImageFile ());
        }
        
        ListIterator pli = pebbleList.listIterator ();
        int pebNum = 1;
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            String pebId = Utils.rightmost ("000"+pebNum,3);
            this.srcData.put ("pebble_"+pebId,peb.serialize ());
            pebNum++;
        }
    }
    
    // updates the src data properties list from the current pebble list and bg image
    // used before storing to make sure everything is saved.
    // NOTE: current deformation are saved AS BASE DATA. That is, a deformed circle 
    // will be stored as an ellipse, not as a circle plus a transformation
    private void updateDeformedSrcData ()
    {
        this.srcData.clear ();
        if ((this.getBgImageFile () != null) && (this.getBgImageFile ().length () > 0))
        {
            this.srcData.put ("bgimage",this.getBgImageFile ());
        }
        
        ListIterator pli = pebbleList.listIterator ();
        int pebNum = 1;
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            String pebId = Utils.rightmost ("000"+pebNum,3);
            this.srcData.put ("pebble_"+pebId,peb.serializeDeformed ());
            pebNum++;
        }
    }

    private void updateFromSrcData ()
    {
        this.pebbleList.clear ();
        this.setBgImageFile ("");
        
        Enumeration srcKeys = srcData.propertyNames ();
        while (srcKeys.hasMoreElements ())
        {
            String theKey = srcKeys.nextElement ().toString ();
            if (theKey.equalsIgnoreCase ("bgimage"))
            {
                setBgImageFile (srcData.getProperty (theKey));
            }
            else if (theKey.toUpperCase ().startsWith ("PEBBLE"))
            {
                pebbleList.add (new Pebble (srcData.getProperty (theKey)));
            }
        }
        
        this.loadBgImage ();
    }
    
    public void loadBgImage ()
    {
        if ((this.getBgImageFile () != null) && (this.getBgImageFile ().length () > 0))
        {
            Toolkit tk = Toolkit.getDefaultToolkit ();
            Image img = null;
            try
            {
                MediaTracker m = new MediaTracker (new JPanel ());
                img = tk.getImage (this.getBgImageFile ());
                m.addImage (img, 0);
                m.waitForAll ();
                this.setOrigBgImage (new BufferedImage (this.DIMX, this.DIMY, BufferedImage.TYPE_INT_RGB));
                this.getOrigBgImage ().createGraphics ().drawImage (img,0,0,null);
                this.setBgImage (new BufferedImage (this.DIMX, this.DIMY, BufferedImage.TYPE_INT_RGB));
                this.getBgImage ().createGraphics ().drawImage (img,0,0,null);
            }
            catch (Exception e)
            {
                // might need a more graceful failure here...
                e.printStackTrace ();
            }
        }
    }
    
    private void setupCrossSectionImage ()
    {
        Graphics2D g2D = this.getCrossSectionImage ().createGraphics ();
        
        g2D.setColor (this.bgColor);
        g2D.fillRect (0,0,this.DIMX,this.DIMY);
        if ((this.getBgImage () != null) && this.isShowBackgroundImage ())
        {
            g2D.drawImage (this.getBgImage (),null,0,0);
        }
        else
        {
            g2D.drawImage (this.bgCrossImage,null,0,0);
        }
        this.strainEllipse.showOn (g2D);
        
        ListIterator pli = pebbleList.listIterator ();
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            peb.showSelectionOn (g2D);
            peb.showOn (g2D);
            if (this.isAdminMode ())
            {
                peb.showHandleOn (g2D);
            }
        }
    }
    
    public void transformMe (AffineTransform transformer)
    {
        this.setCurrentDeformation (transformer);
        
        if (this.getBgImage () != null)
        {
            Graphics2D bgG2D = this.getBgImage ().createGraphics ();
            bgG2D.setColor (Color.WHITE);
            bgG2D.fillRect (0,0,this.DIMX,this.DIMY);
            bgG2D.drawImage ((Image)(this.getOrigBgImage ()),transformer,null);
        }
        
        Graphics2D bgXG2D = this.bgCrossImage.createGraphics ();
        bgXG2D.setColor (Color.WHITE);
        bgXG2D.fillRect (0,0,this.DIMX,this.DIMY);
        bgXG2D.drawImage ((Image)(this.origBgCrossImage),transformer,null);
        
        this.maxLongRatio = 0;
        this.maxShortRatio = 0;
        double ratio = 0.0;
        ListIterator pli = pebbleList.listIterator ();
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            peb.transformMe (transformer);
            
            ratio = peb.getRatioLongToShort ();
            if (ratio > this.maxLongRatio)
            { this.maxLongRatio = ratio; }
            
            ratio = peb.getRatioShortToLong ();
            if (ratio > this.maxShortRatio)
            { this.maxShortRatio = ratio; }
        }
        this.strainEllipse.transformMe (transformer);
        this.setupCrossSectionImage ();

        this.updateWatchingPlottersXMax (maxLongRatio);
    }
    
    public void updateWatchingPlottersXMax(double newXMax)
    {
        ListIterator wpi = this.watchingPlotters.listIterator ();
        while (wpi.hasNext ())
        {
            PebblePlotter pp = (PebblePlotter)(wpi.next ());
            if (! pp.isManualXMax ())
            {
                pp.setXMaxValAuto (newXMax);
            }
            pp.repaint ();
        }
    }
    
    public void setPebbleAxesState (boolean axesState)
    {
        ListIterator pli = pebbleList.listIterator ();
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            peb.setShowAxes (axesState);
        }
        this.setupCrossSectionImage ();
        this.setPebbleShowAxesState (axesState);
    }
    
    public void setPebblesFillState (boolean fillState)
    {
        ListIterator pli = pebbleList.listIterator ();
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            peb.setFillEllipse (fillState);
        }
        this.setupCrossSectionImage ();
        this.setPebbleFillState (fillState);
    }
    
    // resets all relevant transformation variables
    public void reset ()
    {
        this.setOffsetX (0);
        this.setOffsetY (0);
        
        this.setShearX (0);
        this.setShearY (0);
        
        this.setShearAmount (0);
        this.setShearAngle (0);
        
        this.setScaleX (1);
        this.setScaleY (1);
        
        this.setDisplayAngle (0);
        
        this.setMagnifierScale (1.0);
    }
    
    public boolean checkForClickOnPebble (java.awt.event.MouseEvent evt)
    {
        boolean changed = false;
        this.oneOrMorePebblesSelected = false;
        ListIterator pli = pebbleList.listIterator ();
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            if (peb.getCenter ().distance (evt.getX (),evt.getY ()) < Pebble.SELECTION_RADIUS)
            {
                peb.setSelected (true);
                changed = true;
            }
            else if (! evt.isControlDown ())
            {
                peb.setSelected (false);
                changed = true;
            }
            this.oneOrMorePebblesSelected = this.oneOrMorePebblesSelected || peb.isSelected ();
        }
        return changed;
    }
    
    public boolean removeSelectedPebbles ()
    {
        boolean changed = false;
        for (int i = pebbleList.size ()-1; i >= 0; i--)
        {
            if (((Pebble)(pebbleList.get (i))).isSelected ())
            {
                pebbleList.remove (i);
                changed = true;
            }
        }
        return changed;
    }
    
    public void colorSelectedPebbles (Color c)
    {
        for (int i = pebbleList.size ()-1; i >= 0; i--)
        {
            Pebble p = (Pebble) pebbleList.get (i);
            if (p.isSelected ())
            {
                p.setPebbleColor (c);
            }
        }
    }
    
    //**************************************************************************
    // ACCESSOR METHODS
    //**************************************************************************
    
    public BufferedImage getBgImage ()
    {
        return this.bgImage;
    }
    
    public void setBgImage (BufferedImage bgImage)
    {
        this.bgImage = bgImage;
    }
    
    public BufferedImage getCrossSectionImage ()
    {
        return this.crossSectionImage;
    }
    
    public void setCrossSectionImage (BufferedImage crossSectionImage)
    {
        this.crossSectionImage = crossSectionImage;
    }
    
    public double getShearX ()
    {
        return this.shearX;
    }
    
    public void setShearX (double shearX)
    {
        if (shearX > this.MAX_SHEAR_X)
        {
            shearX = this.MAX_SHEAR_X;
        }
        else if (shearX < this.MIN_SHEAR_X)
        {
            shearX = this.MIN_SHEAR_X;
        }
        this.shearX = shearX;
    }
    
    public double getShearY ()
    {
        return this.shearY;
    }
    
    public void setShearY (double shearY)
    {
        if (shearY > this.MAX_SHEAR_Y)
        {
            shearY = this.MAX_SHEAR_Y;
        }
        else if (shearY < this.MIN_SHEAR_Y)
        {
            shearY = this.MIN_SHEAR_Y;
        }
        this.shearY = shearY;
    }
    
    private String getBgImageFile ()
    {
        return this.bgImageFile;
    }
    
    public void setBgImageFile (String bgImageFile)
    {
        this.bgImageFile = bgImageFile;
    }
    
    public BufferedImage getOrigBgImage ()
    {
        return this.origBgImage;
    }
    
    public void setOrigBgImage (BufferedImage origBgImage)
    {
        this.origBgImage = origBgImage;
    }
    
    public boolean isPebbleShowAxesState ()
    {
        return this.pebbleShowAxesState;
    }
    
    public void setPebbleShowAxesState (boolean pebbleShowAxesState)
    {
        this.pebbleShowAxesState = pebbleShowAxesState;
    }
    
    public boolean isPebbleFillState ()
    {
        return this.pebbleFillState;
    }
    
    public void setPebbleFillState (boolean pebbleFillState)
    {
        this.pebbleFillState = pebbleFillState;
    }
    
    public boolean isShowBackgroundImage ()
    {
        return this.showBackgroundImage;
    }
    
    public void setShowBackgroundImage (boolean showBackgroundImage)
    {
        this.showBackgroundImage = showBackgroundImage;
    }
    
    public double getOffsetX ()
    {
        return this.offsetX;
    }
    
    public void setOffsetX (double offsetX)
    {
        this.offsetX = offsetX;
    }
    
    public double getOffsetY ()
    {
        return this.offsetY;
    }
    
    public void setOffsetY (double offsetY)
    {
        this.offsetY = offsetY;
    }
    
    public void addOffsetX (double offsetX)
    {
        this.offsetX += offsetX;
    }
    
    public void addOffsetY (double offsetY)
    {
        this.offsetY += offsetY;
    }
    
    public ArrayList getPebbleList ()
    {
        return pebbleList;
    }
    
    public ArrayList getWatchingPlotters ()
    {
        return watchingPlotters;
    }
    
    public void addWatchingPlotter (PebblePlotter watchingPlotter)
    {
        this.watchingPlotters.add (watchingPlotter);
    }
    
    public double getShearAngle ()
    {
        return shearAngle;
    }
    
    public void setShearAngle (double shearAngle)
    {
        this.shearAngle = shearAngle;
    }
    
    public double getShearAmount ()
    {
        return shearAmount;
    }
    
    public void setShearAmount (double shearAmount)
    {
        this.shearAmount = shearAmount;
    }
    
    public double getScaleX ()
    {
        return scaleX;
    }
    
    public void setScaleX (double scaleX)
    {
        // this.scaleX = scaleX;
        if (scaleX > this.MAX_SCALE_X)
        {
            scaleX = this.MAX_SCALE_X;
        }
        else if (scaleX < this.MIN_SCALE_X)
        {
            scaleX = this.MIN_SCALE_X;
        }
        this.scaleX = scaleX;
        
    }
    
    public double getScaleY ()
    {
        return scaleY;
    }
    
    public void setScaleY (double scaleY)
    {
        //this.scaleY = scaleY;
        if (scaleY > this.MAX_SCALE_Y)
        {
            scaleY = this.MAX_SCALE_Y;
        }
        else if (scaleY < this.MIN_SCALE_Y)
        {
            scaleY = this.MIN_SCALE_Y;
        }
        this.scaleY = scaleY;
    }
    
    public double getMagnifierScale ()
    {
        return magnifierScale;
    }
    
    public void setMagnifierScale (double magnifierScale)
    {
        this.magnifierScale = magnifierScale;
    }
    
    public double getDisplayAngle ()
    {
        return displayAngle;
    }
    
    public void setDisplayAngle (double displayAngle)
    {
        while (displayAngle > Math.PI)
        {
            displayAngle -= (Math.PI * 2.0) + .000001;
        }
        while (displayAngle < (Math.PI * -1.0))
        {
            displayAngle += (Math.PI * 2.0);
        }
        //displayAngle = ((double)((int)(displayAngle * 1000.0)))/1000.0;
        displayAngle = Utils.truncForDisplay (displayAngle); //(double)((int)(displayAngle * Math.pow (10.0,(double)Pebble.DISPLAY_PRECISION))))/Math.pow (10.0,(double)Pebble.DISPLAY_PRECISION);
        this.displayAngle = displayAngle;
    }
    
    public AffineTransform getCurrentDeformation ()
    {
        return currentDeformation;
    }
    
    public void setCurrentDeformation (AffineTransform currentDeformation)
    {
        this.currentDeformation = currentDeformation;
    }
    
    public boolean isAdminMode ()
    {
        return adminMode;
    }
    
    public void setAdminMode (boolean adminMode)
    {
        this.adminMode = adminMode;
        /* 2007/06/05 - removed the de-selecting so we can use selection elsewhere
        if (! adminMode)
        { // deselect all pebbles
            ListIterator pli = pebbleList.listIterator ();
            while (pli.hasNext ())
            {
                Pebble peb = (Pebble)(pli.next ());
                peb.setSelected (false);
            }
        }*/
    }
    
    public boolean isOneOrMorePebblesSelected ()
    {
        return oneOrMorePebblesSelected;
    }
    
    public void setOneOrMorePebblesSelected (boolean oneOrMorePebblesSelected)
    {
        this.oneOrMorePebblesSelected = oneOrMorePebblesSelected;
    }

    public StrainPebble getStrainEllipse ()
    {
        return strainEllipse;
    }

    public void setStrainEllipse (StrainPebble strainEllipse)
    {
        this.strainEllipse = strainEllipse;
    }
    
}
