/*
 * MainWindow.java
 *
 * Created on March 7, 2007, 4:39 PM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.geom.Line2D;
import java.io.File;
import java.io.IOException;
import java.util.ListIterator;
import javax.imageio.ImageIO;
import javax.swing.JColorChooser;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * This is the real heart of the GeoShear program, the place from which all the 
 * action is coordinated. The main sections are the cross section, the controls, 
 * and the subordinate windows (plots, help, about, dialogs, etc.). The various 
 * mouse and control analysis / calculations are done in here. The actual deforming 
 * etc. are passed off to the relevant objects.
 * @author cwarren
 */
public class MainWindow extends javax.swing.JFrame
{
    
    private HelpWindow helpWin;
    private AboutWindow aboutWin;
    private BigPlotWindow bigPlotWin;
    private BigPolarWindow bigPolarWin;
    //private StrainEllipseWindow strainEllipseWin;
    //private ImageFormatWindow imageFormatWin;
    
    private CrossSectionDisplay CrossSectionPanel;
    private CrossSection xSect;
    
    private PlotRfVsPhi rfVsPhiPlot;
    private PlotRfVsPhi bigRfVsPhiPlot;
    
    private PebblePlotterPolar rfVs2PhiPolar;
    
    // mouse positions, used to determine drag effects
    private int mouseOrigX;
    private int mouseOrigY;
    private int mouseLatestX;
    private int mouseLatestY;
    private double mouseOrigAngle;
    
    // used in determining shear changes
    private double origShearX;
    private double origShearY;
    private double origScaleX;
    private double origScaleY;
    private double origDisplayAngle;
    
    // used in dragging the cross section
    private int origOffsetX;
    private int origOffsetY;
    
    // coefficient to flip drag effects depending on where the mouse starts
    private double xDirCoeffShear = 1.0;
    private double yDirCoeffShear = 1.0;
    private double xDirCoeffScale = 1.0;
    private double yDirCoeffScale = 1.0;
    private double xDirCoeffShift = 1.0;
    private double yDirCoeffShift = 1.0;
    
    // affects how quickly shearing effects happen when the mouse is used
    private static double shearChange = .0025;
    private static double scaleChange = .0025;
    
    // for saves and loads
    private final JFileChooser fileChooser = new JFileChooser ();
    private final FileFilterImage filterImage = new FileFilterImage();
    private final FileFilterTab filterTab = new FileFilterTab();
    private final FileFilterGeoShear filterGeoShear = new FileFilterGeoShear();
    
    private int areaPreserveState = 0;
    
    private static String LONG_AXIS_LABEL_BASE = "Long Axis: ";
    private static String SHORT_AXIS_LABEL_BASE = "Short Axis: ";
    private static String RF_LABEL_BASE = "Long/Short Ratio: ";
    private static String PHI_LABEL_BASE = "Phi (rad , deg): ";

    /**
     * Represents a kind of area preservation to use when doing a pure shear 
     * deformation - area is not preserved
     */
    public static int AREA_PRESERVE_STATE_NONE = 0;
    /**
     * Represents a kind of area preservation to use when doing a pure shear 
     * deformation - area is preserved, with the Y value of the shear being 
     * automatically adjusted based on the user's modification of the X value
     */
    public static int AREA_PRESERVE_X_INDEPENDENT = 1;
    /**
     * Represents a kind of area preservation to use when doing a pure shear 
     * deformation - area is preserved, with the X value of the shear being 
     * automatically adjusted based on the user's modification of the Y value
     */
    public static int AREA_PRESERVE_Y_INDEPENDENT = 2;
    
    private static String areaButtonTextNone = "Area Not Preserved";
    private static String areaButtonTextX    = "Area Preserved, X ind. var.";
    private static String areaButtonTextY    = "Area Preserved, Y ind. var.";
        
    /** Creates new form MainWindow */
    public MainWindow ()
    {
        initComponents ();
        
        myInitComponents ();
    }
    
    private void myInitComponents ()
    {        
        helpWin = new HelpWindow ();
        aboutWin = new AboutWindow ();
        
        this.setXSect (new CrossSection ());
        
        CrossSectionPanel = new CrossSectionDisplay (getXSect ());
        
        CrossSectionPanel.setBorder (new javax.swing.border.LineBorder (Color.BLUE));
        CrossSectionPanel.setBackground (Color.GRAY);
        CrossSectionPanel.setAlignmentX (0.0F);
        CrossSectionPanel.setAlignmentY (0.0F);
        CrossSectionPanel.setMaximumSize (new java.awt.Dimension (CrossSection.DIMX, CrossSection.DIMY));
        CrossSectionPanel.setMinimumSize (new java.awt.Dimension (CrossSection.DIMX, CrossSection.DIMY));
        CrossSectionPanel.setName ("CrossSectionDisplayPanel");
        CrossSectionPanel.setPreferredSize (new java.awt.Dimension (CrossSection.DIMX, CrossSection.DIMY));
        CrossSectionPanel.setDoubleBuffered (true);
        
        MainDisplay.add (CrossSectionPanel);
        
        //--------------------------
        rfVsPhiPlot = new PlotRfVsPhi ();
        rfVsPhiPlot.setBackground (Color.WHITE);
        rfVsPhiPlot.setAlignmentX (0.0F);
        rfVsPhiPlot.setAlignmentY (0.0F);
        rfVsPhiPlot.setMaximumSize (new java.awt.Dimension (this.RFVsPhiPanel.getWidth (), this.RFVsPhiPanel.getHeight ()));
        rfVsPhiPlot.setMinimumSize (new java.awt.Dimension (this.RFVsPhiPanel.getWidth (), this.RFVsPhiPanel.getHeight ()));
        rfVsPhiPlot.setPreferredSize (new java.awt.Dimension (this.RFVsPhiPanel.getWidth (), this.RFVsPhiPanel.getHeight ()));
        rfVsPhiPlot.setSize (new java.awt.Dimension (this.RFVsPhiPanel.getWidth (), this.RFVsPhiPanel.getHeight ()));
        rfVsPhiPlot.setName ("RfVsPhiPlot");
        rfVsPhiPlot.setDoubleBuffered (true);
        
        rfVsPhiPlot.setUpPlot ();
        rfVsPhiPlot.setPebbleList (this.getXSect ().getPebbleList ());
        
        rfVsPhiPlot.setTitle ("RF vs Phi");
        rfVsPhiPlot.setYMaxValLabel ("90");
        rfVsPhiPlot.setYLabel ("Phi (deg)");
        rfVsPhiPlot.setYMinValLabel ("-90");
        rfVsPhiPlot.setXMinValLabel ("0");
        rfVsPhiPlot.setXLabel ("Rf");
        rfVsPhiPlot.setXMaxValLabel ("More");
        
        this.getXSect ().addWatchingPlotter (rfVsPhiPlot);
        
        RFVsPhiPanel.add (rfVsPhiPlot);
        
        //--------------------------
        bigPlotWin = new BigPlotWindow ();
        PebblePlotterCartesian bigRfVsPhiPlot = new PlotRfVsPhi ();
        
        bigRfVsPhiPlot.setBackground (Color.WHITE);
        bigRfVsPhiPlot.setAlignmentX (0.0F);
        bigRfVsPhiPlot.setAlignmentY (0.0F);
        bigRfVsPhiPlot.setName ("BigRfVsPhiPlot");
        bigRfVsPhiPlot.setDoubleBuffered (true);
        
        bigRfVsPhiPlot.setUpPlot ();
        bigRfVsPhiPlot.setPebbleList (this.getXSect ().getPebbleList ());
        
        bigRfVsPhiPlot.setTitle ("RF vs Phi");
        bigRfVsPhiPlot.setYMaxValLabel ("90");
        bigRfVsPhiPlot.setYLabel ("Phi (deg)");
        bigRfVsPhiPlot.setYMinValLabel ("-90");
        bigRfVsPhiPlot.setXMinValLabel ("0");
        bigRfVsPhiPlot.setXLabel ("Rf");
        bigRfVsPhiPlot.setXMaxValLabel ("3.0");
        bigRfVsPhiPlot.setXMaxValAuto (3.0);
        bigRfVsPhiPlot.setXMaxValManual (3.0);
        
        bigRfVsPhiPlot.setPlotFont (Font.decode ("Arial-PLAIN-12"));
        bigRfVsPhiPlot.setPlotTitleFont (Font.decode ("Arial-BOLD-14"));
        bigRfVsPhiPlot.setTextAllowance (22);
        
        this.getXSect ().addWatchingPlotter (bigRfVsPhiPlot);
        
        bigPlotWin.setPlot (bigRfVsPhiPlot);
        bigPlotWin.setSize (800,600);
        bigPlotWin.setVisible (false);       
        
        //--------------------------
        bigPolarWin = new BigPolarWindow ();
        
        PlotPolarLnRfVs2Phi rfVs2PhiPolar = new PlotPolarLnRfVs2Phi ();
        
        rfVs2PhiPolar.setBackground (Color.WHITE);
        rfVs2PhiPolar.setAlignmentX (0.0F);
        rfVs2PhiPolar.setAlignmentY (0.0F);
        rfVs2PhiPolar.setName ("RfVs2PhiPolar");
        rfVs2PhiPolar.setDoubleBuffered (true);
        
        rfVs2PhiPolar.setUpPlot ();
        rfVs2PhiPolar.setPebbleList (this.getXSect ().getPebbleList ());
        
        rfVs2PhiPolar.setTitle ("ln RF vs 2*Phi");
        
        rfVs2PhiPolar.setPlotFont (Font.decode ("Arial-PLAIN-12"));
        rfVs2PhiPolar.setPlotTitleFont (Font.decode ("Arial-BOLD-14"));
        rfVs2PhiPolar.setTextAllowance (22);
        
        this.getXSect ().addWatchingPlotter (rfVs2PhiPolar);

        bigPolarWin.setPlot (rfVs2PhiPolar);
        bigPolarWin.setSize (600,640);
        bigPolarWin.setVisible (false);
 
        //--------------------------
        //strainEllipseWin = new StrainEllipseWindow ();
        this.getXSect ().getStrainEllipse ().setWatcher (this);
        this.updateStrainInfoFromPebble (this.getXSect ().getStrainEllipse ());
        //blah
        //strainEllipseWin.setVisible (false);
        
        //--------------------------
        this.NewPebbleColorSamplePanel.setBackground (this.CrossSectionPanel.getNewPebColor ().darker ());       
     }
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        DragActionButtonGroup = new javax.swing.ButtonGroup();
        MainPanel = new javax.swing.JPanel();
        MainDisplay = new javax.swing.JPanel();
        jLabel13 = new javax.swing.JLabel();
        ControlsPanel = new javax.swing.JPanel();
        controlsTopRowPanel = new javax.swing.JPanel();
        LoadButton = new javax.swing.JButton();
        jLabel2 = new javax.swing.JLabel();
        AboutButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        HelpButton = new javax.swing.JButton();
        ModifyPebblesToggleButton = new javax.swing.JToggleButton();
        jLabel14 = new javax.swing.JLabel();
        controlsDisplayEditor = new javax.swing.JPanel();
        LoadBackgroundButton = new javax.swing.JButton();
        NewPebbleColorLabel = new javax.swing.JLabel();
        NewPebbleColorSamplePanel = new javax.swing.JPanel();
        controlsDisplayFlags = new javax.swing.JPanel();
        ShowPebbleAxesCheckbox = new javax.swing.JCheckBox();
        jLabel3 = new javax.swing.JLabel();
        FillPebblesCheckbox = new javax.swing.JCheckBox();
        jLabel4 = new javax.swing.JLabel();
        ShowBackgroundCheckbox = new javax.swing.JCheckBox();
        controlsZoom = new javax.swing.JPanel();
        ZoomLabel = new javax.swing.JLabel();
        MagnificationSlider1 = new javax.swing.JSlider();
        jLabel15 = new javax.swing.JLabel();
        controlsResets = new javax.swing.JPanel();
        ResetButton = new javax.swing.JButton();
        jLabel5 = new javax.swing.JLabel();
        RecenterButton = new javax.swing.JButton();
        jLabel6 = new javax.swing.JLabel();
        ResetMagnificationButton = new javax.swing.JButton();
        controlsDeformations = new javax.swing.JPanel();
        SimpleRadioButton1 = new javax.swing.JRadioButton();
        PureRadioButton2 = new javax.swing.JRadioButton();
        RotateRadioButton3 = new javax.swing.JRadioButton();
        XShearLabel = new javax.swing.JLabel();
        YShearLabel = new javax.swing.JLabel();
        XShearTextField = new javax.swing.JTextField();
        YShearTextField = new javax.swing.JTextField();
        AreaPreserveStateButton = new javax.swing.JButton();
        XStretchTextField = new javax.swing.JTextField();
        YStretchTextField = new javax.swing.JTextField();
        AngleRadLabel = new javax.swing.JLabel();
        AngleDegLabel = new javax.swing.JLabel();
        DisplayAngleRadTextField = new javax.swing.JTextField();
        DisplayAngleDegTextField = new javax.swing.JTextField();
        RFVsPhiPanel = new javax.swing.JPanel();
        jLabel16 = new javax.swing.JLabel();
        controlsOtherWindows = new javax.swing.JPanel();
        BigPlotButton = new javax.swing.JButton();
        jLabel7 = new javax.swing.JLabel();
        PolarPlotButton = new javax.swing.JButton();
        jLabel8 = new javax.swing.JLabel();
        jLabel18 = new javax.swing.JLabel();
        controlsSaves = new javax.swing.JPanel();
        SaveButton = new javax.swing.JButton();
        jLabel9 = new javax.swing.JLabel();
        SaveDeformedButton = new javax.swing.JButton();
        jLabel10 = new javax.swing.JLabel();
        jLabel17 = new javax.swing.JLabel();
        controlsExports = new javax.swing.JPanel();
        ExportButton = new javax.swing.JButton();
        jLabel11 = new javax.swing.JLabel();
        ExportDeformedButton = new javax.swing.JButton();
        jLabel12 = new javax.swing.JLabel();
        jLabel19 = new javax.swing.JLabel();
        controlsCameraExit = new javax.swing.JPanel();
        SnapshotButton = new javax.swing.JButton();
        jLabel21 = new javax.swing.JLabel();
        ExitButton = new javax.swing.JButton();
        jLabel20 = new javax.swing.JLabel();
        StrainInfoPanel = new javax.swing.JPanel();
        TitleLabel = new javax.swing.JLabel();
        LongAxisLabel = new javax.swing.JLabel();
        ShortAxisLabel = new javax.swing.JLabel();
        RFLabel = new javax.swing.JLabel();
        PhiLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("GeoShear");
        setName("MainFrame");
        setResizable(false);
        MainPanel.setLayout(new javax.swing.BoxLayout(MainPanel, javax.swing.BoxLayout.X_AXIS));

        MainPanel.setEnabled(false);
        MainPanel.setMaximumSize(new java.awt.Dimension(960, 700));
        MainPanel.setMinimumSize(new java.awt.Dimension(960, 700));
        MainPanel.setPreferredSize(new java.awt.Dimension(960, 700));
        MainDisplay.setLayout(new javax.swing.BoxLayout(MainDisplay, javax.swing.BoxLayout.X_AXIS));

        MainDisplay.setBackground(new java.awt.Color(255, 255, 255));
        MainDisplay.setAlignmentX(0.0F);
        MainDisplay.setMaximumSize(new java.awt.Dimension(700, 700));
        MainDisplay.setMinimumSize(new java.awt.Dimension(700, 700));
        MainDisplay.setPreferredSize(new java.awt.Dimension(700, 700));
        MainDisplay.addMouseMotionListener(new java.awt.event.MouseMotionAdapter()
        {
            public void mouseDragged(java.awt.event.MouseEvent evt)
            {
                MainDisplayMouseDragged(evt);
            }
        });
        MainDisplay.addKeyListener(new java.awt.event.KeyAdapter()
        {
            public void keyReleased(java.awt.event.KeyEvent evt)
            {
                MainDisplayKeyReleased(evt);
            }
            public void keyTyped(java.awt.event.KeyEvent evt)
            {
                MainDisplayKeyTyped(evt);
            }
        });
        MainDisplay.addMouseListener(new java.awt.event.MouseAdapter()
        {
            public void mouseClicked(java.awt.event.MouseEvent evt)
            {
                MainDisplayMouseClicked(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt)
            {
                MainDisplayMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt)
            {
                MainDisplayMouseReleased(evt);
            }
        });

        MainPanel.add(MainDisplay);

        jLabel13.setText(" ");
        jLabel13.setAlignmentY(0.0F);
        MainPanel.add(jLabel13);

        ControlsPanel.setLayout(new javax.swing.BoxLayout(ControlsPanel, javax.swing.BoxLayout.Y_AXIS));

        ControlsPanel.setAlignmentX(0.0F);
        ControlsPanel.setAlignmentY(0.0F);
        ControlsPanel.setMaximumSize(new java.awt.Dimension(260, 700));
        ControlsPanel.setMinimumSize(new java.awt.Dimension(260, 457));
        ControlsPanel.setPreferredSize(new java.awt.Dimension(260, 100));
        controlsTopRowPanel.setLayout(new javax.swing.BoxLayout(controlsTopRowPanel, javax.swing.BoxLayout.X_AXIS));

        controlsTopRowPanel.setAlignmentX(0.0F);
        controlsTopRowPanel.setAlignmentY(0.0F);
        controlsTopRowPanel.setMaximumSize(new java.awt.Dimension(260, 27));
        controlsTopRowPanel.setMinimumSize(new java.awt.Dimension(260, 27));
        controlsTopRowPanel.setPreferredSize(new java.awt.Dimension(260, 27));
        LoadButton.setText("Load Ellipse File");
        LoadButton.setAlignmentY(0.0F);
        LoadButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                LoadButtonActionPerformed(evt);
            }
        });

        controlsTopRowPanel.add(LoadButton);

        jLabel2.setText(" ");
        jLabel2.setAlignmentY(0.0F);
        controlsTopRowPanel.add(jLabel2);

        AboutButton.setText("About");
        AboutButton.setAlignmentY(0.0F);
        AboutButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                AboutButtonActionPerformed(evt);
            }
        });

        controlsTopRowPanel.add(AboutButton);

        jLabel1.setText(" ");
        jLabel1.setAlignmentY(0.0F);
        controlsTopRowPanel.add(jLabel1);

        HelpButton.setText("Help");
        HelpButton.setAlignmentY(0.0F);
        HelpButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                HelpButtonActionPerformed(evt);
            }
        });

        controlsTopRowPanel.add(HelpButton);

        ControlsPanel.add(controlsTopRowPanel);

        ModifyPebblesToggleButton.setText("Display Editor - click to edit");
        ModifyPebblesToggleButton.setAlignmentY(0.0F);
        ModifyPebblesToggleButton.setMaximumSize(new java.awt.Dimension(260, 23));
        ModifyPebblesToggleButton.setMinimumSize(new java.awt.Dimension(260, 23));
        ModifyPebblesToggleButton.setPreferredSize(new java.awt.Dimension(260, 23));
        ModifyPebblesToggleButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                ModifyPebblesToggleButtonActionPerformed(evt);
            }
        });

        ControlsPanel.add(ModifyPebblesToggleButton);

        jLabel14.setText(" ");
        jLabel14.setAlignmentY(0.0F);
        jLabel14.setMaximumSize(new java.awt.Dimension(3, 3));
        jLabel14.setMinimumSize(new java.awt.Dimension(3, 3));
        jLabel14.setPreferredSize(new java.awt.Dimension(3, 3));
        ControlsPanel.add(jLabel14);

        controlsDisplayEditor.setLayout(new javax.swing.BoxLayout(controlsDisplayEditor, javax.swing.BoxLayout.X_AXIS));

        controlsDisplayEditor.setAlignmentX(0.0F);
        controlsDisplayEditor.setAlignmentY(0.0F);
        controlsDisplayEditor.setMaximumSize(new java.awt.Dimension(260, 27));
        controlsDisplayEditor.setMinimumSize(new java.awt.Dimension(260, 27));
        controlsDisplayEditor.setPreferredSize(new java.awt.Dimension(260, 27));
        LoadBackgroundButton.setText("Load Background");
        LoadBackgroundButton.setEnabled(false);
        LoadBackgroundButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                LoadBackgroundButtonActionPerformed(evt);
            }
        });

        controlsDisplayEditor.add(LoadBackgroundButton);

        NewPebbleColorLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        NewPebbleColorLabel.setText("  Pebble Color:");
        NewPebbleColorLabel.setEnabled(false);
        NewPebbleColorLabel.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        controlsDisplayEditor.add(NewPebbleColorLabel);

        NewPebbleColorSamplePanel.setBackground(new java.awt.Color(0, 0, 204));
        NewPebbleColorSamplePanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        NewPebbleColorSamplePanel.setEnabled(false);
        NewPebbleColorSamplePanel.addMouseListener(new java.awt.event.MouseAdapter()
        {
            public void mouseClicked(java.awt.event.MouseEvent evt)
            {
                NewPebbleColorSamplePanelMouseClicked(evt);
            }
        });

        org.jdesktop.layout.GroupLayout NewPebbleColorSamplePanelLayout = new org.jdesktop.layout.GroupLayout(NewPebbleColorSamplePanel);
        NewPebbleColorSamplePanel.setLayout(NewPebbleColorSamplePanelLayout);
        NewPebbleColorSamplePanelLayout.setHorizontalGroup(
            NewPebbleColorSamplePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 71, Short.MAX_VALUE)
        );
        NewPebbleColorSamplePanelLayout.setVerticalGroup(
            NewPebbleColorSamplePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 25, Short.MAX_VALUE)
        );
        controlsDisplayEditor.add(NewPebbleColorSamplePanel);

        ControlsPanel.add(controlsDisplayEditor);

        controlsDisplayFlags.setLayout(new javax.swing.BoxLayout(controlsDisplayFlags, javax.swing.BoxLayout.X_AXIS));

        controlsDisplayFlags.setAlignmentX(0.0F);
        controlsDisplayFlags.setAlignmentY(0.0F);
        controlsDisplayFlags.setMaximumSize(new java.awt.Dimension(260, 20));
        controlsDisplayFlags.setMinimumSize(new java.awt.Dimension(260, 20));
        controlsDisplayFlags.setPreferredSize(new java.awt.Dimension(260, 20));
        ShowPebbleAxesCheckbox.setText("Pebble Axes");
        ShowPebbleAxesCheckbox.setAlignmentY(0.0F);
        ShowPebbleAxesCheckbox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        ShowPebbleAxesCheckbox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        ShowPebbleAxesCheckbox.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                ShowPebbleAxesCheckboxActionPerformed(evt);
            }
        });

        controlsDisplayFlags.add(ShowPebbleAxesCheckbox);

        jLabel3.setText(" ");
        jLabel3.setAlignmentY(0.0F);
        controlsDisplayFlags.add(jLabel3);

        FillPebblesCheckbox.setText("Fill Pebbles");
        FillPebblesCheckbox.setAlignmentY(0.0F);
        FillPebblesCheckbox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        FillPebblesCheckbox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        FillPebblesCheckbox.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                FillPebblesCheckboxActionPerformed(evt);
            }
        });

        controlsDisplayFlags.add(FillPebblesCheckbox);

        jLabel4.setText(" ");
        jLabel4.setAlignmentY(0.0F);
        controlsDisplayFlags.add(jLabel4);

        ShowBackgroundCheckbox.setSelected(true);
        ShowBackgroundCheckbox.setText("Background");
        ShowBackgroundCheckbox.setAlignmentY(0.0F);
        ShowBackgroundCheckbox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        ShowBackgroundCheckbox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        ShowBackgroundCheckbox.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                ShowBackgroundCheckboxActionPerformed(evt);
            }
        });

        controlsDisplayFlags.add(ShowBackgroundCheckbox);

        ControlsPanel.add(controlsDisplayFlags);

        controlsZoom.setLayout(new javax.swing.BoxLayout(controlsZoom, javax.swing.BoxLayout.X_AXIS));

        controlsZoom.setAlignmentX(0.0F);
        controlsZoom.setAlignmentY(0.0F);
        controlsZoom.setMaximumSize(new java.awt.Dimension(260, 32));
        controlsZoom.setMinimumSize(new java.awt.Dimension(260, 32));
        controlsZoom.setPreferredSize(new java.awt.Dimension(260, 32));
        ZoomLabel.setFont(new java.awt.Font("Tahoma", 1, 12));
        ZoomLabel.setText("Zoom:");
        controlsZoom.add(ZoomLabel);

        MagnificationSlider1.setMajorTickSpacing(10);
        MagnificationSlider1.setPaintTicks(true);
        MagnificationSlider1.setAlignmentX(0.0F);
        MagnificationSlider1.addChangeListener(new javax.swing.event.ChangeListener()
        {
            public void stateChanged(javax.swing.event.ChangeEvent evt)
            {
                MagnificationSlider1StateChanged(evt);
            }
        });

        controlsZoom.add(MagnificationSlider1);

        ControlsPanel.add(controlsZoom);

        jLabel15.setText(" ");
        jLabel15.setAlignmentY(0.0F);
        jLabel15.setMaximumSize(new java.awt.Dimension(3, 3));
        jLabel15.setMinimumSize(new java.awt.Dimension(3, 3));
        jLabel15.setPreferredSize(new java.awt.Dimension(3, 3));
        ControlsPanel.add(jLabel15);

        controlsResets.setLayout(new javax.swing.BoxLayout(controlsResets, javax.swing.BoxLayout.X_AXIS));

        controlsResets.setAlignmentX(0.0F);
        controlsResets.setAlignmentY(0.0F);
        controlsResets.setMaximumSize(new java.awt.Dimension(260, 27));
        controlsResets.setMinimumSize(new java.awt.Dimension(260, 27));
        controlsResets.setPreferredSize(new java.awt.Dimension(260, 27));
        ResetButton.setText("Reset");
        ResetButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                ResetButtonActionPerformed(evt);
            }
        });

        controlsResets.add(ResetButton);

        jLabel5.setText(" ");
        jLabel5.setAlignmentY(0.0F);
        controlsResets.add(jLabel5);

        RecenterButton.setText("Re-center");
        RecenterButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                RecenterButtonActionPerformed(evt);
            }
        });

        controlsResets.add(RecenterButton);

        jLabel6.setText(" ");
        jLabel6.setAlignmentY(0.0F);
        controlsResets.add(jLabel6);

        ResetMagnificationButton.setText("Un-Zoom");
        ResetMagnificationButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                ResetMagnificationButtonActionPerformed(evt);
            }
        });

        controlsResets.add(ResetMagnificationButton);

        ControlsPanel.add(controlsResets);

        controlsDeformations.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        controlsDeformations.setAlignmentX(0.0F);
        controlsDeformations.setAlignmentY(0.0F);
        controlsDeformations.setMaximumSize(new java.awt.Dimension(260, 100));
        controlsDeformations.setMinimumSize(new java.awt.Dimension(260, 100));
        controlsDeformations.setPreferredSize(new java.awt.Dimension(260, 100));
        DragActionButtonGroup.add(SimpleRadioButton1);
        SimpleRadioButton1.setSelected(true);
        SimpleRadioButton1.setText("Simple");
        SimpleRadioButton1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        SimpleRadioButton1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        SimpleRadioButton1.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                SimpleRadioButton1ActionPerformed(evt);
            }
        });

        controlsDeformations.add(SimpleRadioButton1, new org.netbeans.lib.awtextra.AbsoluteConstraints(24, 8, -1, -1));

        DragActionButtonGroup.add(PureRadioButton2);
        PureRadioButton2.setText("Pure");
        PureRadioButton2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        PureRadioButton2.setMargin(new java.awt.Insets(0, 0, 0, 0));
        PureRadioButton2.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                PureRadioButton2ActionPerformed(evt);
            }
        });

        controlsDeformations.add(PureRadioButton2, new org.netbeans.lib.awtextra.AbsoluteConstraints(88, 8, -1, -1));

        DragActionButtonGroup.add(RotateRadioButton3);
        RotateRadioButton3.setText("Rotate");
        RotateRadioButton3.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        RotateRadioButton3.setMargin(new java.awt.Insets(0, 0, 0, 0));
        RotateRadioButton3.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                RotateRadioButton3ActionPerformed(evt);
            }
        });

        controlsDeformations.add(RotateRadioButton3, new org.netbeans.lib.awtextra.AbsoluteConstraints(168, 8, -1, -1));

        XShearLabel.setFont(new java.awt.Font("Tahoma", 0, 12));
        XShearLabel.setText("X: ");
        controlsDeformations.add(XShearLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(8, 24, -1, -1));

        YShearLabel.setFont(new java.awt.Font("Tahoma", 0, 12));
        YShearLabel.setText("Y: ");
        controlsDeformations.add(YShearLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(8, 48, -1, -1));

        XShearTextField.setFont(new java.awt.Font("Tahoma", 1, 12));
        XShearTextField.setHorizontalAlignment(javax.swing.JTextField.LEFT);
        XShearTextField.setText("0.0");
        XShearTextField.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                XShearTextFieldActionPerformed(evt);
            }
        });
        XShearTextField.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                XShearTextFieldFocusLost(evt);
            }
        });
        XShearTextField.addKeyListener(new java.awt.event.KeyAdapter()
        {
            public void keyPressed(java.awt.event.KeyEvent evt)
            {
                XShearTextFieldKeyPressed(evt);
            }
            public void keyReleased(java.awt.event.KeyEvent evt)
            {
                XShearTextFieldKeyReleased(evt);
            }
        });

        controlsDeformations.add(XShearTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(24, 24, 48, -1));

        YShearTextField.setFont(new java.awt.Font("Tahoma", 1, 12));
        YShearTextField.setHorizontalAlignment(javax.swing.JTextField.LEFT);
        YShearTextField.setText("0.0");
        YShearTextField.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                YShearTextFieldActionPerformed(evt);
            }
        });
        YShearTextField.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                YShearTextFieldFocusLost(evt);
            }
        });
        YShearTextField.addKeyListener(new java.awt.event.KeyAdapter()
        {
            public void keyReleased(java.awt.event.KeyEvent evt)
            {
                YShearTextFieldKeyReleased(evt);
            }
        });

        controlsDeformations.add(YShearTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(24, 48, 48, -1));

        AreaPreserveStateButton.setFont(new java.awt.Font("Tahoma", 0, 10));
        AreaPreserveStateButton.setText("Area Not Preserved (click to change)");
        AreaPreserveStateButton.setEnabled(false);
        AreaPreserveStateButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                AreaPreserveStateButtonActionPerformed(evt);
            }
        });

        controlsDeformations.add(AreaPreserveStateButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(8, 72, 216, -1));

        XStretchTextField.setFont(new java.awt.Font("Tahoma", 1, 12));
        XStretchTextField.setText("1.0");
        XStretchTextField.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                XStretchTextFieldActionPerformed(evt);
            }
        });
        XStretchTextField.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                XStretchTextFieldFocusLost(evt);
            }
        });
        XStretchTextField.addKeyListener(new java.awt.event.KeyAdapter()
        {
            public void keyReleased(java.awt.event.KeyEvent evt)
            {
                XStretchTextFieldKeyReleased(evt);
            }
        });

        controlsDeformations.add(XStretchTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(88, 24, 48, -1));

        YStretchTextField.setFont(new java.awt.Font("Tahoma", 1, 12));
        YStretchTextField.setText("1.0");
        YStretchTextField.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                YStretchTextFieldActionPerformed(evt);
            }
        });
        YStretchTextField.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                YStretchTextFieldFocusLost(evt);
            }
        });
        YStretchTextField.addKeyListener(new java.awt.event.KeyAdapter()
        {
            public void keyReleased(java.awt.event.KeyEvent evt)
            {
                YStretchTextFieldKeyReleased(evt);
            }
        });

        controlsDeformations.add(YStretchTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(88, 48, 48, -1));

        AngleRadLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        AngleRadLabel.setText("rad:");
        controlsDeformations.add(AngleRadLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(144, 24, -1, -1));

        AngleDegLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        AngleDegLabel.setText("deg:");
        controlsDeformations.add(AngleDegLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(144, 48, -1, -1));

        DisplayAngleRadTextField.setFont(new java.awt.Font("Tahoma", 1, 12));
        DisplayAngleRadTextField.setText("0.0");
        DisplayAngleRadTextField.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                DisplayAngleRadTextFieldActionPerformed(evt);
            }
        });
        DisplayAngleRadTextField.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                DisplayAngleRadTextFieldFocusLost(evt);
            }
        });
        DisplayAngleRadTextField.addKeyListener(new java.awt.event.KeyAdapter()
        {
            public void keyReleased(java.awt.event.KeyEvent evt)
            {
                DisplayAngleRadTextFieldKeyReleased(evt);
            }
        });

        controlsDeformations.add(DisplayAngleRadTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(168, 24, 48, -1));

        DisplayAngleDegTextField.setFont(new java.awt.Font("Tahoma", 1, 12));
        DisplayAngleDegTextField.setText("0");
        DisplayAngleDegTextField.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                DisplayAngleDegTextFieldActionPerformed(evt);
            }
        });
        DisplayAngleDegTextField.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                DisplayAngleDegTextFieldFocusLost(evt);
            }
        });
        DisplayAngleDegTextField.addKeyListener(new java.awt.event.KeyAdapter()
        {
            public void keyReleased(java.awt.event.KeyEvent evt)
            {
                DisplayAngleDegTextFieldKeyReleased(evt);
            }
        });

        controlsDeformations.add(DisplayAngleDegTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(168, 48, 48, -1));

        ControlsPanel.add(controlsDeformations);

        RFVsPhiPanel.setLayout(new javax.swing.BoxLayout(RFVsPhiPanel, javax.swing.BoxLayout.X_AXIS));

        RFVsPhiPanel.setBackground(new java.awt.Color(255, 255, 255));
        RFVsPhiPanel.setAlignmentX(0.0F);
        RFVsPhiPanel.setAlignmentY(0.0F);
        RFVsPhiPanel.setMaximumSize(new java.awt.Dimension(260, 200));
        RFVsPhiPanel.setMinimumSize(new java.awt.Dimension(260, 200));
        RFVsPhiPanel.setPreferredSize(new java.awt.Dimension(260, 200));
        ControlsPanel.add(RFVsPhiPanel);

        jLabel16.setText(" ");
        jLabel16.setAlignmentY(0.0F);
        jLabel16.setMaximumSize(new java.awt.Dimension(3, 3));
        jLabel16.setMinimumSize(new java.awt.Dimension(3, 3));
        jLabel16.setPreferredSize(new java.awt.Dimension(3, 3));
        ControlsPanel.add(jLabel16);

        controlsOtherWindows.setLayout(new javax.swing.BoxLayout(controlsOtherWindows, javax.swing.BoxLayout.X_AXIS));

        controlsOtherWindows.setAlignmentX(0.0F);
        controlsOtherWindows.setAlignmentY(0.0F);
        controlsOtherWindows.setDoubleBuffered(false);
        controlsOtherWindows.setMaximumSize(new java.awt.Dimension(260, 27));
        controlsOtherWindows.setMinimumSize(new java.awt.Dimension(260, 27));
        controlsOtherWindows.setPreferredSize(new java.awt.Dimension(260, 27));
        BigPlotButton.setText("Big Cartesian Plot");
        BigPlotButton.setAlignmentY(0.0F);
        BigPlotButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                BigPlotButtonActionPerformed(evt);
            }
        });

        controlsOtherWindows.add(BigPlotButton);

        jLabel7.setText(" ");
        jLabel7.setAlignmentY(0.0F);
        controlsOtherWindows.add(jLabel7);

        PolarPlotButton.setText("Big Polar Plot");
        PolarPlotButton.setAlignmentY(0.0F);
        PolarPlotButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                PolarPlotButtonActionPerformed(evt);
            }
        });

        controlsOtherWindows.add(PolarPlotButton);

        jLabel8.setText(" ");
        jLabel8.setAlignmentY(0.0F);
        controlsOtherWindows.add(jLabel8);

        ControlsPanel.add(controlsOtherWindows);

        jLabel18.setText(" ");
        jLabel18.setAlignmentY(0.0F);
        jLabel18.setMaximumSize(new java.awt.Dimension(3, 3));
        jLabel18.setMinimumSize(new java.awt.Dimension(3, 3));
        jLabel18.setPreferredSize(new java.awt.Dimension(3, 3));
        ControlsPanel.add(jLabel18);

        controlsSaves.setLayout(new javax.swing.BoxLayout(controlsSaves, javax.swing.BoxLayout.X_AXIS));

        controlsSaves.setAlignmentX(0.0F);
        controlsSaves.setAlignmentY(0.0F);
        controlsSaves.setMaximumSize(new java.awt.Dimension(260, 27));
        controlsSaves.setMinimumSize(new java.awt.Dimension(260, 27));
        controlsSaves.setPreferredSize(new java.awt.Dimension(260, 27));
        SaveButton.setText("Save to .ges");
        SaveButton.setAlignmentX(0.5F);
        SaveButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                SaveButtonActionPerformed(evt);
            }
        });

        controlsSaves.add(SaveButton);

        jLabel9.setText(" ");
        jLabel9.setAlignmentY(0.0F);
        controlsSaves.add(jLabel9);

        SaveDeformedButton.setText("Save Deformed");
        SaveDeformedButton.setAlignmentX(0.5F);
        SaveDeformedButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                SaveDeformedButtonActionPerformed(evt);
            }
        });

        controlsSaves.add(SaveDeformedButton);

        jLabel10.setText(" ");
        jLabel10.setAlignmentY(0.0F);
        controlsSaves.add(jLabel10);

        ControlsPanel.add(controlsSaves);

        jLabel17.setText(" ");
        jLabel17.setAlignmentY(0.0F);
        jLabel17.setMaximumSize(new java.awt.Dimension(3, 3));
        jLabel17.setMinimumSize(new java.awt.Dimension(3, 3));
        jLabel17.setPreferredSize(new java.awt.Dimension(3, 3));
        ControlsPanel.add(jLabel17);

        controlsExports.setLayout(new javax.swing.BoxLayout(controlsExports, javax.swing.BoxLayout.X_AXIS));

        controlsExports.setAlignmentX(0.0F);
        controlsExports.setAlignmentY(0.0F);
        controlsExports.setMaximumSize(new java.awt.Dimension(260, 27));
        controlsExports.setMinimumSize(new java.awt.Dimension(260, 27));
        controlsExports.setPreferredSize(new java.awt.Dimension(260, 27));
        ExportButton.setText("Export to .tab");
        ExportButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                ExportButtonActionPerformed(evt);
            }
        });

        controlsExports.add(ExportButton);

        jLabel11.setText(" ");
        jLabel11.setAlignmentY(0.0F);
        controlsExports.add(jLabel11);

        ExportDeformedButton.setText("Export Deformed");
        ExportDeformedButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                ExportDeformedButtonActionPerformed(evt);
            }
        });

        controlsExports.add(ExportDeformedButton);

        jLabel12.setText(" ");
        jLabel12.setAlignmentY(0.0F);
        controlsExports.add(jLabel12);

        ControlsPanel.add(controlsExports);

        jLabel19.setText(" ");
        jLabel19.setAlignmentY(0.0F);
        jLabel19.setMaximumSize(new java.awt.Dimension(3, 3));
        jLabel19.setMinimumSize(new java.awt.Dimension(3, 3));
        jLabel19.setPreferredSize(new java.awt.Dimension(3, 3));
        ControlsPanel.add(jLabel19);

        controlsCameraExit.setLayout(new javax.swing.BoxLayout(controlsCameraExit, javax.swing.BoxLayout.X_AXIS));

        controlsCameraExit.setAlignmentX(0.0F);
        controlsCameraExit.setAlignmentY(0.0F);
        controlsCameraExit.setMaximumSize(new java.awt.Dimension(260, 40));
        controlsCameraExit.setMinimumSize(new java.awt.Dimension(260, 40));
        controlsCameraExit.setPreferredSize(new java.awt.Dimension(260, 40));
        SnapshotButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_camera_large_40x30.gif")));
        SnapshotButton.setToolTipText("Take a snapshot of the cross section");
        SnapshotButton.setAlignmentY(0.0F);
        SnapshotButton.setMaximumSize(new java.awt.Dimension(70, 39));
        SnapshotButton.setMinimumSize(new java.awt.Dimension(70, 39));
        SnapshotButton.setPreferredSize(new java.awt.Dimension(70, 39));
        SnapshotButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                SnapshotButtonActionPerformed(evt);
            }
        });

        controlsCameraExit.add(SnapshotButton);

        jLabel21.setText("                               ");
        jLabel21.setAlignmentY(0.0F);
        controlsCameraExit.add(jLabel21);

        ExitButton.setFont(new java.awt.Font("Tahoma", 1, 18));
        ExitButton.setText("Exit");
        ExitButton.setAlignmentY(0.0F);
        ExitButton.setMaximumSize(new java.awt.Dimension(70, 39));
        ExitButton.setMinimumSize(new java.awt.Dimension(70, 39));
        ExitButton.setPreferredSize(new java.awt.Dimension(70, 39));
        ExitButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                ExitButtonActionPerformed(evt);
            }
        });

        controlsCameraExit.add(ExitButton);

        ControlsPanel.add(controlsCameraExit);

        jLabel20.setText(" ");
        jLabel20.setAlignmentY(0.0F);
        jLabel20.setMaximumSize(new java.awt.Dimension(3, 3));
        jLabel20.setMinimumSize(new java.awt.Dimension(3, 3));
        jLabel20.setPreferredSize(new java.awt.Dimension(3, 3));
        ControlsPanel.add(jLabel20);

        StrainInfoPanel.setLayout(new javax.swing.BoxLayout(StrainInfoPanel, javax.swing.BoxLayout.Y_AXIS));

        TitleLabel.setFont(new java.awt.Font("Tahoma", 1, 18));
        TitleLabel.setText("Strain Ellipse Info");
        StrainInfoPanel.add(TitleLabel);

        LongAxisLabel.setText("Long Axis:     ");
        StrainInfoPanel.add(LongAxisLabel);

        ShortAxisLabel.setText("Short Axis:     ");
        StrainInfoPanel.add(ShortAxisLabel);

        RFLabel.setText("Long/Short Ratio:     ");
        StrainInfoPanel.add(RFLabel);

        PhiLabel.setText("phi (rad , deg):     ");
        StrainInfoPanel.add(PhiLabel);

        ControlsPanel.add(StrainInfoPanel);

        MainPanel.add(ControlsPanel);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(MainPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 970, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(MainPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void ExportDeformedButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_ExportDeformedButtonActionPerformed
    {//GEN-HEADEREND:event_ExportDeformedButtonActionPerformed
        fileChooser.setFileFilter (this.filterTab);
        int returnVal = fileChooser.showSaveDialog (this);
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            File saveFile = fileChooser.getSelectedFile ();
            if (this.filterTab.accept (saveFile))
            {
                this.getXSect ().exportDeformedToFile (fileChooser.getSelectedFile ());
                try
                {
                    JOptionPane.showMessageDialog (this,"Data exported to "+saveFile.getCanonicalPath ());
                }
                catch (IOException exc)
                {
                    exc.printStackTrace ();
                }                
            } else
            {
                JOptionPane.showMessageDialog (this,"Unsupported format, only "+this.filterTab.getDescription ()+". Save aborted.");
            }            
        }
        this.repaint ();
    }//GEN-LAST:event_ExportDeformedButtonActionPerformed

    private void DisplayAngleDegTextFieldKeyReleased (java.awt.event.KeyEvent evt)//GEN-FIRST:event_DisplayAngleDegTextFieldKeyReleased
    {//GEN-HEADEREND:event_DisplayAngleDegTextFieldKeyReleased
        try 
        {
            double origValue = Double.parseDouble (DisplayAngleDegTextField.getText ());
            double newValue =  this.calcFieldValueOnArrowKey(origValue,evt,1);
            //JOptionPane.showMessageDialog (this,"old="+origValue+"; new="+newValue);
            this.getXSect ().setDisplayAngle (-1.0 * Math.toRadians (newValue));
            this.updateAngleTextFields ();
            DisplayAngleDegTextField.setText (Utils.truncTextDecimal (""+newValue,0));
            this.repaint ();
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
    }//GEN-LAST:event_DisplayAngleDegTextFieldKeyReleased

    private void DisplayAngleRadTextFieldKeyReleased (java.awt.event.KeyEvent evt)//GEN-FIRST:event_DisplayAngleRadTextFieldKeyReleased
    {//GEN-HEADEREND:event_DisplayAngleRadTextFieldKeyReleased
        try 
        {
            double origValue = Double.parseDouble (DisplayAngleRadTextField.getText ());
            double newValue =  this.calcFieldValueOnArrowKey(origValue,evt,.01);
            this.getXSect ().setDisplayAngle (-1.0 * newValue);
            this.updateAngleTextFields ();
            //DisplayAngleRadTextField.setText (Double.toString (Utils.truncForDisplay (newValue)));
            this.repaint ();
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
    }//GEN-LAST:event_DisplayAngleRadTextFieldKeyReleased

    private void YStretchTextFieldKeyReleased (java.awt.event.KeyEvent evt)//GEN-FIRST:event_YStretchTextFieldKeyReleased
    {//GEN-HEADEREND:event_YStretchTextFieldKeyReleased
        try 
        {
            double origValue = Double.parseDouble (YStretchTextField.getText ());
            double newValue =  this.calcFieldValueOnArrowKey(origValue,evt,.01);
            this.getXSect ().setScaleY (newValue);
            YStretchTextField.setText (Double.toString (Utils.truncForDisplay (newValue)));
            this.repaint ();
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
    }//GEN-LAST:event_YStretchTextFieldKeyReleased

    private void XStretchTextFieldKeyReleased (java.awt.event.KeyEvent evt)//GEN-FIRST:event_XStretchTextFieldKeyReleased
    {//GEN-HEADEREND:event_XStretchTextFieldKeyReleased
        try 
        {
            double origValue = Double.parseDouble (XStretchTextField.getText ());
            double newValue =  this.calcFieldValueOnArrowKey(origValue,evt,.01);
            this.getXSect ().setScaleX (newValue);
            XStretchTextField.setText (Double.toString (Utils.truncForDisplay (newValue)));
            this.repaint ();
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
    }//GEN-LAST:event_XStretchTextFieldKeyReleased

    private void YShearTextFieldKeyReleased (java.awt.event.KeyEvent evt)//GEN-FIRST:event_YShearTextFieldKeyReleased
    {//GEN-HEADEREND:event_YShearTextFieldKeyReleased
        try 
        {
            double origValue = Double.parseDouble (YShearTextField.getText ());
            double newValue =  this.calcFieldValueOnArrowKey(origValue,evt,.01);
            this.getXSect ().setShearY (newValue);
            YShearTextField.setText (Double.toString (Utils.truncForDisplay (newValue)));
            this.repaint ();
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
    }//GEN-LAST:event_YShearTextFieldKeyReleased

    private void XShearTextFieldKeyReleased (java.awt.event.KeyEvent evt)//GEN-FIRST:event_XShearTextFieldKeyReleased
    {//GEN-HEADEREND:event_XShearTextFieldKeyReleased
        try 
        {
            double origValue = Double.parseDouble (XShearTextField.getText ());
            double newValue =  this.calcFieldValueOnArrowKey(origValue,evt,.01);
            this.getXSect ().setShearX (newValue);
            XShearTextField.setText (Double.toString (Utils.truncForDisplay (newValue)));
            this.repaint ();
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
    }//GEN-LAST:event_XShearTextFieldKeyReleased
   
    private double calcFieldValueOnArrowKey(double origValue, java.awt.event.KeyEvent evt, double increment)
    {
        double newValue = origValue;

        // NOTE: to deal with stupid floating-point math issues
        double largeIncrement = increment * 1.01;
        double smallIncrement = increment * .99;

        if (evt.getKeyCode () == java.awt.event.KeyEvent.VK_DOWN)
        {
            if (origValue <= 0)
            {
                newValue =  Utils.truncForDisplay (origValue - largeIncrement);
            } else
            {
                newValue = Utils.truncForDisplay (origValue - smallIncrement);
            }
        } else if (evt.getKeyCode () == java.awt.event.KeyEvent.VK_UP)
        {
            if (origValue < 0)
            {
                newValue = Utils.truncForDisplay (origValue + smallIncrement);
            } else
            {
                newValue = Utils.truncForDisplay (origValue + largeIncrement);
            }
        }
        return newValue;
    }
    
    private void XShearTextFieldKeyPressed (java.awt.event.KeyEvent evt)//GEN-FIRST:event_XShearTextFieldKeyPressed
    {//GEN-HEADEREND:event_XShearTextFieldKeyPressed
    }//GEN-LAST:event_XShearTextFieldKeyPressed

    private void PolarPlotButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_PolarPlotButtonActionPerformed
    {//GEN-HEADEREND:event_PolarPlotButtonActionPerformed
        bigPolarWin.getPlot ().setUpPlot ();
        bigPolarWin.setVisible (true);
    }//GEN-LAST:event_PolarPlotButtonActionPerformed

    private void SnapshotButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_SnapshotButtonActionPerformed
    {//GEN-HEADEREND:event_SnapshotButtonActionPerformed
        fileChooser.setFileFilter (this.filterImage);
        int returnVal = fileChooser.showSaveDialog (this);
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            try
            {
                File saveFile = fileChooser.getSelectedFile ();
                if (this.filterImage.accept (saveFile))
                {
                    ImageIO.write (this.getXSect ().getCrossSectionImage (),Utils.rightmost (saveFile.getName (),3),saveFile);
                    JOptionPane.showMessageDialog (this,"Image saved to "+saveFile.getCanonicalPath ());
                } else
                {
                    JOptionPane.showMessageDialog (this,"Unsupported format, only "+this.filterImage.getDescription ()+". Save aborted.");
                }
            }
            catch (IOException exc)
            {
                exc.printStackTrace ();
            }
        }
        this.repaint ();
    }//GEN-LAST:event_SnapshotButtonActionPerformed

    private void ExportButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_ExportButtonActionPerformed
    {//GEN-HEADEREND:event_ExportButtonActionPerformed
        fileChooser.setFileFilter (this.filterTab);
        int returnVal = fileChooser.showSaveDialog (this);
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            File saveFile = fileChooser.getSelectedFile ();
            if (this.filterTab.accept (saveFile))
            {
                this.getXSect ().exportToFile (fileChooser.getSelectedFile ());
                try
                {
                    JOptionPane.showMessageDialog (this,"Data exported to "+saveFile.getCanonicalPath ());
                }
                catch (IOException exc)
                {
                    exc.printStackTrace ();
                }                
            } else
            {
                JOptionPane.showMessageDialog (this,"Unsupported format, only "+this.filterTab.getDescription ()+". Save aborted.");
            }            
        }
        this.repaint ();
    }//GEN-LAST:event_ExportButtonActionPerformed

    private void MainDisplayMouseReleased (java.awt.event.MouseEvent evt)//GEN-FIRST:event_MainDisplayMouseReleased
    {//GEN-HEADEREND:event_MainDisplayMouseReleased
        if (this.ModifyPebblesToggleButton.isSelected ()
            && this.CrossSectionPanel.isAddingPebble ()
            && (this.CrossSectionPanel.getNewPebLongLen () < Pebble.MIN_LONG_AXIS))
        {
            this.clearNewPebbleCreation ();
            this.repaint ();
        }
    }//GEN-LAST:event_MainDisplayMouseReleased

    private void SaveDeformedButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_SaveDeformedButtonActionPerformed
    {//GEN-HEADEREND:event_SaveDeformedButtonActionPerformed
        fileChooser.setFileFilter (this.filterGeoShear);
        int returnVal = fileChooser.showSaveDialog (this);
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            File saveFile = fileChooser.getSelectedFile ();
            if (this.filterGeoShear.accept (saveFile))
            {
                this.getXSect ().storeDeformedToFile (saveFile);
                try
                {
                    JOptionPane.showMessageDialog (this,"Saved to "+saveFile.getCanonicalPath ());
                }
                catch (IOException exc)
                {
                    exc.printStackTrace ();
                }                
            } else
            {
                JOptionPane.showMessageDialog (this,"Unsupported format, only "+this.filterGeoShear.getDescription ()+". Save aborted.");
            }            
        }
        this.repaint ();
    }//GEN-LAST:event_SaveDeformedButtonActionPerformed
    
    private void AreaPreserveStateButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_AreaPreserveStateButtonActionPerformed
    {//GEN-HEADEREND:event_AreaPreserveStateButtonActionPerformed
        int initState = this.getAreaPreserveState ();
        if (initState == this.AREA_PRESERVE_STATE_NONE)
        {
            this.AreaPreserveStateButton.setText (this.areaButtonTextY);
            this.setAreaPreserveState (this.AREA_PRESERVE_Y_INDEPENDENT);
            this.XStretchTextField.setEnabled (false);
            this.YStretchTextField.setEnabled (true);
        }
        else if (initState == this.AREA_PRESERVE_Y_INDEPENDENT)
        {
            this.AreaPreserveStateButton.setText (this.areaButtonTextX);
            this.setAreaPreserveState ( this.AREA_PRESERVE_X_INDEPENDENT);
            this.YStretchTextField.setEnabled (false);
            this.XStretchTextField.setEnabled (true);
        }
        else if (initState == this.AREA_PRESERVE_X_INDEPENDENT)
        {
            this.AreaPreserveStateButton.setText (this.areaButtonTextNone);
            this.setAreaPreserveState (this.AREA_PRESERVE_STATE_NONE);
            this.XStretchTextField.setEnabled (true);
            this.YStretchTextField.setEnabled (true);
        }
        this.repaint ();
    }//GEN-LAST:event_AreaPreserveStateButtonActionPerformed
    
    private void RotateRadioButton3ActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_RotateRadioButton3ActionPerformed
    {//GEN-HEADEREND:event_RotateRadioButton3ActionPerformed
        this.AreaPreserveStateButton.setEnabled (false);
    }//GEN-LAST:event_RotateRadioButton3ActionPerformed
    
    private void SimpleRadioButton1ActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_SimpleRadioButton1ActionPerformed
    {//GEN-HEADEREND:event_SimpleRadioButton1ActionPerformed
        this.AreaPreserveStateButton.setEnabled (false);
    }//GEN-LAST:event_SimpleRadioButton1ActionPerformed
    
    private void PureRadioButton2ActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_PureRadioButton2ActionPerformed
    {//GEN-HEADEREND:event_PureRadioButton2ActionPerformed
        this.AreaPreserveStateButton.setEnabled (true);
    }//GEN-LAST:event_PureRadioButton2ActionPerformed
    
    private void DisplayAngleRadTextFieldFocusLost (java.awt.event.FocusEvent evt)//GEN-FIRST:event_DisplayAngleRadTextFieldFocusLost
    {//GEN-HEADEREND:event_DisplayAngleRadTextFieldFocusLost
        this.DisplayAngleRadTextAction ();
    }//GEN-LAST:event_DisplayAngleRadTextFieldFocusLost
    
    private void DisplayAngleDegTextFieldFocusLost (java.awt.event.FocusEvent evt)//GEN-FIRST:event_DisplayAngleDegTextFieldFocusLost
    {//GEN-HEADEREND:event_DisplayAngleDegTextFieldFocusLost
        this.DisplayAngleDegTextAction ();
    }//GEN-LAST:event_DisplayAngleDegTextFieldFocusLost
    
    private void YStretchTextFieldFocusLost (java.awt.event.FocusEvent evt)//GEN-FIRST:event_YStretchTextFieldFocusLost
    {//GEN-HEADEREND:event_YStretchTextFieldFocusLost
        this.YStretchTextAction ();
    }//GEN-LAST:event_YStretchTextFieldFocusLost
    
    private void XStretchTextFieldFocusLost (java.awt.event.FocusEvent evt)//GEN-FIRST:event_XStretchTextFieldFocusLost
    {//GEN-HEADEREND:event_XStretchTextFieldFocusLost
        this.XStretchTextAction ();
    }//GEN-LAST:event_XStretchTextFieldFocusLost
    
    private void YShearTextFieldFocusLost (java.awt.event.FocusEvent evt)//GEN-FIRST:event_YShearTextFieldFocusLost
    {//GEN-HEADEREND:event_YShearTextFieldFocusLost
        this.YShearTextAction ();
    }//GEN-LAST:event_YShearTextFieldFocusLost
    
    private void XShearTextFieldFocusLost (java.awt.event.FocusEvent evt)//GEN-FIRST:event_XShearTextFieldFocusLost
    {//GEN-HEADEREND:event_XShearTextFieldFocusLost
        this.XShearTextAction ();
    }//GEN-LAST:event_XShearTextFieldFocusLost
    
    private void DisplayAngleDegTextFieldActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_DisplayAngleDegTextFieldActionPerformed
    {//GEN-HEADEREND:event_DisplayAngleDegTextFieldActionPerformed
        this.DisplayAngleDegTextAction ();
    }//GEN-LAST:event_DisplayAngleDegTextFieldActionPerformed
    
    private void DisplayAngleDegTextAction ()
    {
        try
        {
            this.getXSect ().setDisplayAngle (Math.toRadians (-1.0 * Double.parseDouble (this.DisplayAngleDegTextField.getText ())));
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
        this.updateAngleTextFields ();
        this.repaint ();
        //blah
    }
    
    private void updateAngleTextFields ()
    {
        DisplayAngleRadTextField.setText (Double.toString (-1.0 * this.getXSect ().getDisplayAngle ()));
        DisplayAngleDegTextField.setText (Integer.toString ((int)(Math.toDegrees (-1.0 * this.getXSect ().getDisplayAngle ()) + (this.getXSect ().getDisplayAngle ()>=0?-.5:.5) )));
    }
        
    private void NewPebbleColorSamplePanelMouseClicked (java.awt.event.MouseEvent evt)//GEN-FIRST:event_NewPebbleColorSamplePanelMouseClicked
    {//GEN-HEADEREND:event_NewPebbleColorSamplePanelMouseClicked
        if (this.NewPebbleColorSamplePanel.isEnabled ())
        {
            if (evt.isAltDown ())
            {
                this.getXSect ().colorSelectedPebbles (this.NewPebbleColorSamplePanel.getBackground ());
            }
            else
            {
                Color newColor = JColorChooser.showDialog (this,"Pebble Color Chooser",this.CrossSectionPanel.getNewPebColor ());
                if (newColor != null)
                {
                    this.CrossSectionPanel.setNewPebColor (newColor);
                    this.NewPebbleColorSamplePanel.setBackground (newColor);
                }
            }
            
            this.repaint ();
        }
    }//GEN-LAST:event_NewPebbleColorSamplePanelMouseClicked
    
    private void ModifyPebblesToggleButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_ModifyPebblesToggleButtonActionPerformed
    {//GEN-HEADEREND:event_ModifyPebblesToggleButtonActionPerformed
        this.getXSect ().setAdminMode (this.ModifyPebblesToggleButton.isSelected ());
        if (this.ModifyPebblesToggleButton.isSelected ())
        {
            this.ModifyPebblesToggleButton.setText ("Crosssection Editor - click to finish");
        } else
        {
            this.ModifyPebblesToggleButton.setText ("Crosssection Editor - click to edit");            
        }
        this.MainDisplay.requestFocusInWindow ();
        this.NewPebbleColorLabel.setEnabled (this.ModifyPebblesToggleButton.isSelected ());
        this.NewPebbleColorSamplePanel.setEnabled (this.ModifyPebblesToggleButton.isSelected ());
        this.LoadBackgroundButton.setEnabled (this.ModifyPebblesToggleButton.isSelected ());
        //this.RemoveBackgroundButton.setEnabled (this.ModifyPebblesToggleButton.isSelected ());
        if (this.ModifyPebblesToggleButton.isSelected ())
        {
            this.NewPebbleColorSamplePanel.setBackground (this.CrossSectionPanel.getNewPebColor ());
            this.doResetAction ();
            this.setControlsEnableState (false);
        }
        else
        {
            this.NewPebbleColorSamplePanel.setBackground (this.CrossSectionPanel.getNewPebColor ().darker ());
            this.setControlsEnableState (true);
        }
        this.repaint ();
    }//GEN-LAST:event_ModifyPebblesToggleButtonActionPerformed
    
    private void setControlsEnableState (boolean state)
    {
        this.ResetButton.setEnabled (state);
        this.RecenterButton.setEnabled (state);
        
        this.SimpleRadioButton1.setEnabled (state);
        this.PureRadioButton2.setEnabled (state);
        this.RotateRadioButton3.setEnabled (state);
        this.AreaPreserveStateButton.setEnabled (state && this.PureRadioButton2.isSelected ());
        
        this.XShearTextField.setEnabled (state);
        this.XStretchTextField.setEnabled (state && (this.getAreaPreserveState () != this.AREA_PRESERVE_Y_INDEPENDENT));
        this.YShearTextField.setEnabled (state);
        this.YStretchTextField.setEnabled (state && (this.getAreaPreserveState () != this.AREA_PRESERVE_X_INDEPENDENT));
        this.DisplayAngleRadTextField.setEnabled (state);
        this.DisplayAngleDegTextField.setEnabled (state);
        
        this.SnapshotButton.setEnabled (state);

        this.ResetMagnificationButton.setEnabled (state);
        this.MagnificationSlider1.setEnabled (state);
        
        this.SaveButton.setEnabled (state);
        this.LoadButton.setEnabled (state);
        this.SaveDeformedButton.setEnabled (state);
        this.ExportButton.setEnabled (state);
    }
    
    private void clearNewPebbleCreation()
    {
        this.CrossSectionPanel.setSettingLongAxis (false);
        this.CrossSectionPanel.setSettingShortAxis (false);
        this.CrossSectionPanel.setAddingPebble (false);
        this.CrossSectionPanel.setNewPeb (null);
    }
    
    private void MainDisplayKeyTyped (java.awt.event.KeyEvent evt)//GEN-FIRST:event_MainDisplayKeyTyped
    {//GEN-HEADEREND:event_MainDisplayKeyTyped
        if (this.ModifyPebblesToggleButton.isSelected ()) // admin/modify behavior
        {
            if (evt.isControlDown () && evt.getKeyChar () == '\u0001')
            {
                ListIterator pli = this.getXSect ().getPebbleList ().listIterator ();
                while (pli.hasNext ())
                {
                    Pebble p = (Pebble)(pli.next ());
                    p.setSelected (true);
                }
                this.getXSect ().setOneOrMorePebblesSelected (true);
                this.repaint ();
            } else
            {
                if (this.CrossSectionPanel.isAddingPebble ()) // admin/modify behavior
                {
                    if (this.CrossSectionPanel.getNewPebLongLen () < Pebble.MIN_LONG_AXIS)
                    {
                        this.clearNewPebbleCreation ();
                    } else
                    {
                        if (this.CrossSectionPanel.isSettingLongAxis ())
                        {
                            this.CrossSectionPanel.setSettingLongAxis (false);
                            this.CrossSectionPanel.setSettingShortAxis (true);
                            this.CrossSectionPanel.setNewPebEnd (new Point (this.mouseLatestX,this.mouseLatestY));
                            if (this.CrossSectionPanel.getNewPebLongLen () < Pebble.MIN_LONG_AXIS)
                            {
                                this.clearNewPebbleCreation ();
                            }
                        }
                        else if (this.CrossSectionPanel.isSettingShortAxis ())
                        {
                            this.CrossSectionPanel.setSettingShortAxis (false);
                            this.CrossSectionPanel.setAddingPebble (false);
                            int confirmRes =
                                    JOptionPane.showOptionDialog (this,"Do you want to add this pebble?","Pebble add confirmation",
                                    JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,
                                    null,null,null);
                            if (confirmRes == JOptionPane.OK_OPTION)
                            {
                                this.CrossSectionPanel.getNewPeb ().setFillEllipse (this.FillPebblesCheckbox.isSelected ());
                                this.CrossSectionPanel.getNewPeb ().setShowAxes (this.ShowPebbleAxesCheckbox.isSelected ());
                                this.CrossSectionPanel.getNewPeb ().setSelected (false);
                                this.getXSect ().getPebbleList ().add (this.CrossSectionPanel.getNewPeb ());
                            }

                            this.clearNewPebbleCreation ();
                        }
                    }
                    this.repaint ();
                }
            }
        }
    }//GEN-LAST:event_MainDisplayKeyTyped
    
    private void MainDisplayKeyReleased (java.awt.event.KeyEvent evt)//GEN-FIRST:event_MainDisplayKeyReleased
    {//GEN-HEADEREND:event_MainDisplayKeyReleased
        if (this.ModifyPebblesToggleButton.isSelected ())
        {
            if ((evt.getKeyCode () == KeyEvent.VK_BACK_SPACE)
            || (evt.getKeyCode () == KeyEvent.VK_DELETE))
            {
                if (this.getXSect ().isOneOrMorePebblesSelected ())
                {
                    //int confirmRes = JOptionPane.showConfirmDialog (this,"Are you sure you want to delete?");
                    int confirmRes =
                            JOptionPane.showOptionDialog (this,"Are you sure you want to delete?","Pebble delete confirmation",
                            JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,
                            null,null,null);
                    if (confirmRes == JOptionPane.OK_OPTION)
                    {
                        if (this.getXSect ().removeSelectedPebbles ())
                        {
                            this.repaint ();
                        }
                    }
                }
            }
        }
    }//GEN-LAST:event_MainDisplayKeyReleased
    
    
    private void MainDisplayMouseClicked (java.awt.event.MouseEvent evt)//GEN-FIRST:event_MainDisplayMouseClicked
    {//GEN-HEADEREND:event_MainDisplayMouseClicked
        if (this.ModifyPebblesToggleButton.isSelected ())
        {
            if (this.getXSect ().checkForClickOnPebble (evt))
            {
                this.repaint ();
            }
        }
    }//GEN-LAST:event_MainDisplayMouseClicked
    
    private void LoadBackgroundButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_LoadBackgroundButtonActionPerformed
    {//GEN-HEADEREND:event_LoadBackgroundButtonActionPerformed
        fileChooser.setFileFilter (this.filterImage);
        int returnVal = fileChooser.showOpenDialog (this);
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            // load the cross section from the file
            this.getXSect ().setBgImageFile (fileChooser.getSelectedFile ().getAbsolutePath ());
            this.getXSect ().loadBgImage ();
            
            // and reset the various options
            this.ShowBackgroundCheckbox.setSelected (true);
            this.getXSect ().setShowBackgroundImage (true);
            
            // show success message?
        }
        this.repaint ();
    }//GEN-LAST:event_LoadBackgroundButtonActionPerformed
        
    private void BigPlotButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_BigPlotButtonActionPerformed
    {//GEN-HEADEREND:event_BigPlotButtonActionPerformed
        bigPlotWin.setVisible (true);
    }//GEN-LAST:event_BigPlotButtonActionPerformed
    
    private void DisplayAngleRadTextFieldActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_DisplayAngleRadTextFieldActionPerformed
    {//GEN-HEADEREND:event_DisplayAngleRadTextFieldActionPerformed
        this.DisplayAngleRadTextAction ();
    }//GEN-LAST:event_DisplayAngleRadTextFieldActionPerformed
    
    private void DisplayAngleRadTextAction ()
    {
        try
        {
            this.getXSect ().setDisplayAngle (-1.0 * Double.parseDouble (this.DisplayAngleRadTextField.getText ()));
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
        this.updateAngleTextFields ();
        this.repaint ();
    }
    
    private void YStretchTextFieldActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_YStretchTextFieldActionPerformed
    {//GEN-HEADEREND:event_YStretchTextFieldActionPerformed
        this.YStretchTextAction ();
    }//GEN-LAST:event_YStretchTextFieldActionPerformed
    
    private void YStretchTextAction ()
    {
        try
        {
            this.getXSect ().setScaleY (Double.parseDouble (YStretchTextField.getText ()));
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
        YStretchTextField.setText (Double.toString (Utils.truncForDisplay (this.getXSect ().getScaleY ())));
        
        if (this.getAreaPreserveState () == this.AREA_PRESERVE_Y_INDEPENDENT)
        {
            try
            {
                this.getXSect ().setScaleX (1.0/Double.parseDouble (YStretchTextField.getText ()));
            }
            catch (NumberFormatException exc)
            { // invalid entries are just ignored
            }
            XStretchTextField.setText (Double.toString (Utils.truncForDisplay (this.getXSect ().getScaleX ())));
        }
        
        this.repaint ();
    }
    
    private void XStretchTextFieldActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_XStretchTextFieldActionPerformed
    {//GEN-HEADEREND:event_XStretchTextFieldActionPerformed
        this.XStretchTextAction ();
    }//GEN-LAST:event_XStretchTextFieldActionPerformed
    
    private void XStretchTextAction ()
    {
        try
        {
            this.getXSect ().setScaleX (Double.parseDouble (XStretchTextField.getText ()));
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
        XStretchTextField.setText (Double.toString (Utils.truncForDisplay (this.getXSect ().getScaleX ())));
        
        if (this.getAreaPreserveState () == this.AREA_PRESERVE_X_INDEPENDENT)
        {
            try
            {
                this.getXSect ().setScaleY (1.0/Double.parseDouble (XStretchTextField.getText ()));
            }
            catch (NumberFormatException exc)
            { // invalid entries are just ignored
            }
            YStretchTextField.setText (Double.toString (Utils.truncForDisplay (this.getXSect ().getScaleY ())));
        }
        
        this.repaint ();
    }
    
    private void MagnificationSlider1StateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_MagnificationSlider1StateChanged
    {//GEN-HEADEREND:event_MagnificationSlider1StateChanged
        int amt = this.MagnificationSlider1.getModel ().getValue ();
        if (amt < 50)
        {
            this.getXSect ().setMagnifierScale (((1.0 - this.getXSect ().MIN_MAGNIFIER) * (double)amt/50.0) + this.getXSect ().MIN_MAGNIFIER);
        }
        else
        {
            this.getXSect ().setMagnifierScale ((this.getXSect ().MAX_MAGNIFIER - 1.0) * (((double)amt-50.0)/50.0) + 1.0);
        }
        this.repaint ();
    }//GEN-LAST:event_MagnificationSlider1StateChanged
    
    private void ResetMagnificationButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_ResetMagnificationButtonActionPerformed
    {//GEN-HEADEREND:event_ResetMagnificationButtonActionPerformed
        this.getXSect ().setMagnifierScale (1.0);
        this.MagnificationSlider1.getModel ().setValue (50);
        this.repaint ();
    }//GEN-LAST:event_ResetMagnificationButtonActionPerformed
    
    private void YShearTextFieldActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_YShearTextFieldActionPerformed
    {//GEN-HEADEREND:event_YShearTextFieldActionPerformed
        this.YShearTextAction ();
    }//GEN-LAST:event_YShearTextFieldActionPerformed
    
    private void YShearTextAction ()
    {
        try
        {
            this.getXSect ().setShearY (Utils.truncForDisplay (Double.parseDouble (YShearTextField.getText ())));
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
        YShearTextField.setText (Double.toString (Utils.truncForDisplay (this.getXSect ().getShearY ())));
        this.repaint ();    }
    
    private void XShearTextFieldActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_XShearTextFieldActionPerformed
    {//GEN-HEADEREND:event_XShearTextFieldActionPerformed
        this.XShearTextAction ();
    }//GEN-LAST:event_XShearTextFieldActionPerformed
    
    private void XShearTextAction ()
    {
        try
        {
            this.getXSect ().setShearX (Utils.truncForDisplay (Double.parseDouble (XShearTextField.getText ())));
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
        XShearTextField.setText (Double.toString (Utils.truncForDisplay (this.getXSect ().getShearX ())));
        this.repaint ();
    }
    
    private void RecenterButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_RecenterButtonActionPerformed
    {//GEN-HEADEREND:event_RecenterButtonActionPerformed
        this.getXSect ().setOffsetX (0);
        this.getXSect ().setOffsetY (0);
        this.repaint ();
    }//GEN-LAST:event_RecenterButtonActionPerformed
    
    private void LoadButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_LoadButtonActionPerformed
    {//GEN-HEADEREND:event_LoadButtonActionPerformed
        fileChooser.setFileFilter (this.filterGeoShear);
        int returnVal = fileChooser.showOpenDialog (this);
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            // load the cross section from the file
            this.getXSect ().loadFromFile (fileChooser.getSelectedFile ());
            
            // re-set strain ellipse watcher
            this.getXSect ().getStrainEllipse ().setWatcher (this);
            this.updateStrainInfoFromPebble (this.getXSect ().getStrainEllipse ());
            
            this.doResetAction ();
        }
        this.repaint ();
    }//GEN-LAST:event_LoadButtonActionPerformed
    
    private void SaveButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_SaveButtonActionPerformed
    {//GEN-HEADEREND:event_SaveButtonActionPerformed
        fileChooser.setFileFilter (this.filterGeoShear);
        int returnVal = fileChooser.showSaveDialog (this);
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            File saveFile = fileChooser.getSelectedFile ();
            if (this.filterGeoShear.accept (saveFile))
            {
                this.getXSect ().storeToFile (saveFile);
                try
                {
                    JOptionPane.showMessageDialog (this,"Saved to "+saveFile.getCanonicalPath ());
                }
                catch (IOException exc)
                {
                    exc.printStackTrace ();
                }                
            } else
            {
                JOptionPane.showMessageDialog (this,"Unsupported format, only "+this.filterGeoShear.getDescription ()+". Save aborted.");
            }            
        }
        this.repaint ();
    }//GEN-LAST:event_SaveButtonActionPerformed
    
    private void ShowBackgroundCheckboxActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_ShowBackgroundCheckboxActionPerformed
    {//GEN-HEADEREND:event_ShowBackgroundCheckboxActionPerformed
        this.getXSect ().setShowBackgroundImage (this.ShowBackgroundCheckbox.isSelected ());
        this.repaint ();
    }//GEN-LAST:event_ShowBackgroundCheckboxActionPerformed
    
    private void FillPebblesCheckboxActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_FillPebblesCheckboxActionPerformed
    {//GEN-HEADEREND:event_FillPebblesCheckboxActionPerformed
        this.getXSect ().setPebblesFillState (this.FillPebblesCheckbox.isSelected ());
        this.repaint ();
    }//GEN-LAST:event_FillPebblesCheckboxActionPerformed
    
    private void ShowPebbleAxesCheckboxActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_ShowPebbleAxesCheckboxActionPerformed
    {//GEN-HEADEREND:event_ShowPebbleAxesCheckboxActionPerformed
        this.getXSect ().setPebbleAxesState (this.ShowPebbleAxesCheckbox.isSelected ());
        this.repaint ();
    }//GEN-LAST:event_ShowPebbleAxesCheckboxActionPerformed
    
    private void ResetButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_ResetButtonActionPerformed
    {//GEN-HEADEREND:event_ResetButtonActionPerformed
        this.doResetAction ();
        this.repaint ();
    }//GEN-LAST:event_ResetButtonActionPerformed
    
    private void doResetAction ()
    {
        this.getXSect ().reset ();

        this.FillPebblesCheckbox.setSelected (false);
        this.getXSect ().setPebbleFillState (false);
        this.ShowPebbleAxesCheckbox.setSelected (false);
        this.getXSect ().setPebbleAxesState (false);
        this.ShowBackgroundCheckbox.setSelected (true);
        this.getXSect ().setShowBackgroundImage (true);

        DisplayAngleRadTextField.setText ("0.0");
        DisplayAngleDegTextField.setText ("0");
        this.MagnificationSlider1.getModel ().setValue (50);
        XShearTextField.setText ("0.0");
        YShearTextField.setText ("0.0");
        XStretchTextField.setText ("1.0");
        YStretchTextField.setText ("1.0");
        
    }
    
    private void MainDisplayMousePressed (java.awt.event.MouseEvent evt)//GEN-FIRST:event_MainDisplayMousePressed
    {//GEN-HEADEREND:event_MainDisplayMousePressed
        this.MainDisplay.requestFocusInWindow ();
        this.mouseOrigX = evt.getX ();
        this.mouseOrigY = evt.getY ();

        this.clearNewPebbleCreation ();
        
        this.mouseOrigAngle = Math.atan2 ((double)(this.getXSect ().DIMY)/2.0-(double)this.mouseOrigY,
                (double)this.mouseOrigX-(double)(this.getXSect ().DIMX)/2.0);
        
        this.origOffsetX = (int)(this.getXSect ().getOffsetX ());
        this.origOffsetY = (int)(this.getXSect ().getOffsetY ());
        
        this.origShearX = this.getXSect ().getShearX ();
        this.origShearY = this.getXSect ().getShearY ();
        
        this.origScaleX = this.getXSect ().getScaleX ();
        this.origScaleY = this.getXSect ().getScaleY ();
        
        this.origDisplayAngle = this.getXSect ().getDisplayAngle ();
        
        // set the directional coefficients depending on where the mouse was pressed
        if (this.mouseOrigX < this.MainDisplay.getSize ().width / 2)
        { // left half
            if (this.mouseOrigY < this.MainDisplay.getSize ().height / 2)
            { // upper left
                this.xDirCoeffShear = 1.0;
                this.yDirCoeffShear = 1.0;
                
                this.xDirCoeffScale = 1.0;
                this.yDirCoeffScale = 1.0;
            }
            else
            { // lower left
                this.xDirCoeffShear = -1.0;
                this.yDirCoeffShear = 1.0;
                
                this.xDirCoeffScale = 1.0;
                this.yDirCoeffScale = -1.0;
            }
        }
        else
        { // right half
            if (this.mouseOrigY < this.MainDisplay.getSize ().height / 2)
            { // upper right
                this.xDirCoeffShear = 1.0;
                this.yDirCoeffShear = -1.0;
                
                this.xDirCoeffScale = -1.0;
                this.yDirCoeffScale = 1.0;
            }
            else
            { // lower right
                this.xDirCoeffShear = -1.0;
                this.yDirCoeffShear = -1.0;
                
                this.xDirCoeffScale = -1.0;
                this.yDirCoeffScale = -1.0;
            }
        }
        
        this.xDirCoeffShift = 1.0;
        this.yDirCoeffShift = 1.0;
        
        // holding down limits deformatin to the X direction, shift limits it to Y
        if (evt.isControlDown ())
        {
            this.yDirCoeffShear = 0.0;
            this.yDirCoeffScale = 0.0;
            this.yDirCoeffShift = 0.0;
        }
        if (evt.isShiftDown ())
        {
            this.xDirCoeffShear = 0.0;
            this.xDirCoeffScale = 0.0;
            this.xDirCoeffShift = 0.0;
        }
    }//GEN-LAST:event_MainDisplayMousePressed
        
    private void MainDisplayMouseDragged (java.awt.event.MouseEvent evt)//GEN-FIRST:event_MainDisplayMouseDragged
    {//GEN-HEADEREND:event_MainDisplayMouseDragged
        int newX = evt.getX ();
        int newY = evt.getY ();
        this.mouseLatestX = newX;
        this.mouseLatestY = newY;
        double newMouseOrigAngle = Math.atan2 ((double)(this.getXSect ().DIMY)/2.0-(double)newY,
                (double)newX-(double)(this.getXSect ().DIMX)/2.0);
        
        if (this.ModifyPebblesToggleButton.isSelected ()) // admin/modify behavior
        {
            if (! evt.isControlDown ())
            {
                if (! this.CrossSectionPanel.isAddingPebble ()) // admin/modify behavior
                {
                    this.CrossSectionPanel.setAddingPebble (true);
                    this.CrossSectionPanel.setSettingLongAxis (true);
                    this.CrossSectionPanel.setSettingShortAxis (false);
                }
                
                if (this.CrossSectionPanel.isSettingLongAxis ())
                {
                    this.CrossSectionPanel.setNewPebCenter (new Point ((newX + this.mouseOrigX)/2,(newY + this.mouseOrigY)/2));
                    this.CrossSectionPanel.setNewPebLongLen ((int)evt.getPoint ().distance (this.mouseOrigX,this.mouseOrigY));
                    this.CrossSectionPanel.setNewPebShortLen (1);
                    this.CrossSectionPanel.setNewPebAngle (Math.atan2 ((double)newY - (double)this.mouseOrigY , (double)newX - (double)this.mouseOrigX));
                }
                else if (this.CrossSectionPanel.isSettingShortAxis ())
                {
                    this.CrossSectionPanel.setNewPebShortLen ((int)Math.min (
                            Line2D.ptLineDist (this.mouseOrigX,this.mouseOrigY,this.CrossSectionPanel.getNewPebEnd ().x,this.CrossSectionPanel.getNewPebEnd ().y,evt.getX (),evt.getY ()),
                            (double)this.CrossSectionPanel.getNewPebLongLen ()));
                } 
                
                Pebble newPebble = new Pebble (
                        this.CrossSectionPanel.getNewPebCenter ().x,
                        this.CrossSectionPanel.getNewPebCenter ().y,
                        this.CrossSectionPanel.getNewPebLongLen (),
                        this.CrossSectionPanel.getNewPebShortLen (),
                        this.CrossSectionPanel.getNewPebAngle (),
                        this.CrossSectionPanel.getNewPebColor ());
                newPebble.setShowAxes (true);
                newPebble.setFillEllipse (false);
                newPebble.setSelected (true);
                this.CrossSectionPanel.setNewPeb (newPebble);
            } // end if (! evt.isControlDown ())
        }
        else // normal, deforming behavior - i.e. NOT this.ModifyPebblesToggleButton.isSelected ()
        {
            int diffX = this.mouseOrigX - newX;
            int diffY = this.mouseOrigY - newY;
            double diffAngle = this.mouseOrigAngle - newMouseOrigAngle;
            
            // if the alt key is held drag the cross section, otherwise deform it
            if (evt.isAltDown ())
            {
                this.getXSect ().setOffsetX (origOffsetX + diffX * -1 * xDirCoeffShift);
                this.getXSect ().setOffsetY (origOffsetY + diffY * -1 * yDirCoeffShift);
            }
            else // alt NOT pressed
            {
                if (this.SimpleRadioButton1.isSelected ())
                {
                    double newShearX = this.origShearX + (diffX * this.shearChange * this.xDirCoeffShear);
                    double newShearY = this.origShearY + (diffY * this.shearChange * this.yDirCoeffShear);
                    newShearX = Utils.truncForDisplay (newShearX);
                    newShearY = Utils.truncForDisplay (newShearY);
                    this.getXSect ().setShearX (newShearX);
                    this.getXSect ().setShearY (newShearY);
                    XShearTextField.setText (Double.toString (this.getXSect ().getShearX ()));
                    YShearTextField.setText (Double.toString (this.getXSect ().getShearY ()));
                }
                else if (this.PureRadioButton2.isSelected ())
                {
                    double newScaleX = this.origScaleX + (diffX * this.scaleChange * this.xDirCoeffScale);
                    double newScaleY = this.origScaleY + (diffY * this.scaleChange * this.yDirCoeffScale);
                    
                    int preserveState = this.getAreaPreserveState ();
                    if (preserveState == this.AREA_PRESERVE_STATE_NONE)
                    {
                        this.getXSect ().setScaleX (Utils.truncForDisplay (newScaleX));
                        this.getXSect ().setScaleY (Utils.truncForDisplay (newScaleY));
                    }
                    else if (preserveState == this.AREA_PRESERVE_Y_INDEPENDENT)
                    {
                        this.getXSect ().setScaleY (Utils.truncForDisplay (newScaleY));
                        this.getXSect ().setScaleX (Utils.truncForDisplay (1.0/this.getXSect ().getScaleY ()));
                    }
                    else if (preserveState == this.AREA_PRESERVE_X_INDEPENDENT)
                    {
                        this.getXSect ().setScaleX (Utils.truncForDisplay (newScaleX));
                        this.getXSect ().setScaleY (Utils.truncForDisplay (1.0/this.getXSect ().getScaleX ()));
                    }
                    
                    XStretchTextField.setText (Double.toString (Utils.truncForDisplay (this.getXSect ().getScaleX ())));
                    YStretchTextField.setText (Double.toString (Utils.truncForDisplay (this.getXSect ().getScaleY ())));
                    
                }
                else if (this.RotateRadioButton3.isSelected ())
                {
                    double newDisplayAngle = this.origDisplayAngle + (diffAngle);                    
                    newDisplayAngle = Utils.truncForDisplay(newDisplayAngle); 
                    this.getXSect ().setDisplayAngle (newDisplayAngle);
                    this.updateAngleTextFields ();

                } // end if-tree for which kind of deform to do
            
            } // end alt not pressed
            
        } // end normal, deforming behavior
        
        this.repaint ();
    }//GEN-LAST:event_MainDisplayMouseDragged
    
    private void AboutButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_AboutButtonActionPerformed
    {//GEN-HEADEREND:event_AboutButtonActionPerformed
        aboutWin.setVisible (true);
    }//GEN-LAST:event_AboutButtonActionPerformed
    
    private void HelpButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_HelpButtonActionPerformed
    {//GEN-HEADEREND:event_HelpButtonActionPerformed
        helpWin.setVisible (true);
    }//GEN-LAST:event_HelpButtonActionPerformed
    
    private void ExitButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_ExitButtonActionPerformed
    {//GEN-HEADEREND:event_ExitButtonActionPerformed
        System.exit (0);
    }//GEN-LAST:event_ExitButtonActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main (String args[])
    {
        java.awt.EventQueue.invokeLater (new Runnable ()
        {
            public void run ()
            {
                new MainWindow ().setVisible (true);
            }
        });
    }
    
    /**
     * Gets the CrossSection object which is being displayed
     * @return the CrossSection object which is being displayed
     */
    public CrossSection getXSect ()
    {
        return xSect;
    }
    
    /**
     * Sets the CrossSection object which is being displayed
     * @param xSect the CrossSection object to be displayed
     */
    public void setXSect (CrossSection xSect)
    {
        this.xSect = xSect;
    }
    
    /**
     * Gets the current state of area preservation for pure shear deformations.
     * @return the current state of area preservation for pure shear deformations
     * @see AREA_PRESERVE_STATE_NONE
     * @see AREA_PRESERVE_X_INDEPENDENT
     * @see AREA_PRESERVE_Y_INDEPENDENT
     */
    public int getAreaPreserveState ()
    {
        return areaPreserveState;
    }
    
    /**
     * Sets the current state of area preservation for pure shear deformations.
     * @param areaPreserveState the state of area preservation for pure shear deformations
     * @see AREA_PRESERVE_STATE_NONE
     * @see AREA_PRESERVE_X_INDEPENDENT
     * @see AREA_PRESERVE_Y_INDEPENDENT
     */
    public void setAreaPreserveState (int areaPreserveState)
    {
        if (   (areaPreserveState == MainWindow.AREA_PRESERVE_STATE_NONE)
        || (areaPreserveState == MainWindow.AREA_PRESERVE_X_INDEPENDENT)
        || (areaPreserveState == MainWindow.AREA_PRESERVE_Y_INDEPENDENT))
        {
            this.areaPreserveState = areaPreserveState;
        }
        else
        {
            this.areaPreserveState = MainWindow.AREA_PRESERVE_STATE_NONE;
        }
    }

    /**
     * Gets the window that displays info about the strain ellipse
     * @return the window that displays info about the strain ellipse
     */
    /*public StrainEllipseWindow getStrainEllipseWin ()
    {
        return strainEllipseWin;
    }*/

    /**
     * Sets the window that displays info about the strain ellipse
     * @param strainEllipseWin the window that displays info about the strain ellipse
     */
    /*public void setStrainEllipseWin (StrainEllipseWindow strainEllipseWin)
    {
        this.strainEllipseWin = strainEllipseWin;
    }
*/
    public void updateStrainInfoFromPebble(Pebble peb)
    {
        this.getLongAxisLabel ().setText (MainWindow.LONG_AXIS_LABEL_BASE+Integer.toString (peb.getLongAxisLength ()));
        this.getShortAxisLabel ().setText (MainWindow.SHORT_AXIS_LABEL_BASE+Integer.toString (peb.getShortAxisLength ()));

        String rf = Utils.truncForDisplay (Double.toString ((double) peb.getLongAxisLength () / (double) peb.getShortAxisLength ()));
        this.getRFLabel ().setText (MainWindow.RF_LABEL_BASE+rf);

        String thetaRadString = Utils.truncForDisplay (Double.toString (peb.getThetaRad () * -1.0));
        String thetaDegString = Utils.truncTextDecimal (Double.toString (Math.toDegrees (peb.getThetaRad ()* -1.0)),0);
        
        this.getPhiLabel ().setText (MainWindow.PHI_LABEL_BASE+thetaRadString+" , "+thetaDegString);
    }

    public javax.swing.JLabel getLongAxisLabel ()
    {
        return LongAxisLabel;
    }

    public void setLongAxisLabel (javax.swing.JLabel LongAxisLabel)
    {
        this.LongAxisLabel = LongAxisLabel;
    }

    public javax.swing.JLabel getPhiLabel ()
    {
        return PhiLabel;
    }

    public void setPhiLabel (javax.swing.JLabel PhiLabel)
    {
        this.PhiLabel = PhiLabel;
    }

    public javax.swing.JLabel getRFLabel ()
    {
        return RFLabel;
    }

    public void setRFLabel (javax.swing.JLabel RFLabel)
    {
        this.RFLabel = RFLabel;
    }

    public javax.swing.JLabel getShortAxisLabel ()
    {
        return ShortAxisLabel;
    }

    public void setShortAxisLabel (javax.swing.JLabel ShortAxisLabel)
    {
        this.ShortAxisLabel = ShortAxisLabel;
    }

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton AboutButton;
    private javax.swing.JLabel AngleDegLabel;
    private javax.swing.JLabel AngleRadLabel;
    private javax.swing.JButton AreaPreserveStateButton;
    private javax.swing.JButton BigPlotButton;
    private javax.swing.JPanel ControlsPanel;
    private javax.swing.JTextField DisplayAngleDegTextField;
    private javax.swing.JTextField DisplayAngleRadTextField;
    private javax.swing.ButtonGroup DragActionButtonGroup;
    private javax.swing.JButton ExitButton;
    private javax.swing.JButton ExportButton;
    private javax.swing.JButton ExportDeformedButton;
    private javax.swing.JCheckBox FillPebblesCheckbox;
    private javax.swing.JButton HelpButton;
    private javax.swing.JButton LoadBackgroundButton;
    private javax.swing.JButton LoadButton;
    private javax.swing.JLabel LongAxisLabel;
    private javax.swing.JSlider MagnificationSlider1;
    private javax.swing.JPanel MainDisplay;
    private javax.swing.JPanel MainPanel;
    private javax.swing.JToggleButton ModifyPebblesToggleButton;
    private javax.swing.JLabel NewPebbleColorLabel;
    private javax.swing.JPanel NewPebbleColorSamplePanel;
    private javax.swing.JLabel PhiLabel;
    private javax.swing.JButton PolarPlotButton;
    private javax.swing.JRadioButton PureRadioButton2;
    private javax.swing.JLabel RFLabel;
    private javax.swing.JPanel RFVsPhiPanel;
    private javax.swing.JButton RecenterButton;
    private javax.swing.JButton ResetButton;
    private javax.swing.JButton ResetMagnificationButton;
    private javax.swing.JRadioButton RotateRadioButton3;
    private javax.swing.JButton SaveButton;
    private javax.swing.JButton SaveDeformedButton;
    private javax.swing.JLabel ShortAxisLabel;
    private javax.swing.JCheckBox ShowBackgroundCheckbox;
    private javax.swing.JCheckBox ShowPebbleAxesCheckbox;
    private javax.swing.JRadioButton SimpleRadioButton1;
    private javax.swing.JButton SnapshotButton;
    private javax.swing.JPanel StrainInfoPanel;
    private javax.swing.JLabel TitleLabel;
    private javax.swing.JLabel XShearLabel;
    private javax.swing.JTextField XShearTextField;
    private javax.swing.JTextField XStretchTextField;
    private javax.swing.JLabel YShearLabel;
    private javax.swing.JTextField YShearTextField;
    private javax.swing.JTextField YStretchTextField;
    private javax.swing.JLabel ZoomLabel;
    private javax.swing.JPanel controlsCameraExit;
    private javax.swing.JPanel controlsDeformations;
    private javax.swing.JPanel controlsDisplayEditor;
    private javax.swing.JPanel controlsDisplayFlags;
    private javax.swing.JPanel controlsExports;
    private javax.swing.JPanel controlsOtherWindows;
    private javax.swing.JPanel controlsResets;
    private javax.swing.JPanel controlsSaves;
    private javax.swing.JPanel controlsTopRowPanel;
    private javax.swing.JPanel controlsZoom;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel20;
    private javax.swing.JLabel jLabel21;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    // End of variables declaration//GEN-END:variables
    
}
